/*------------------------------------------------------------------------------------------*\
 *   Copyright (C) 2006,2007, ... ,2011 AVM GmbH <fritzbox_info@avm.de>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
\*------------------------------------------------------------------------------------------*/


#include <linux/proc_fs.h>
#include <linux/seq_file.h>
#include <linux/sched.h>
#include "cpmac_if.h"
#include "cpmac_debug.h"
#if defined(CONFIG_AR9)
#include <asm/mach-infineon/ifx_regs.h>
#include "ifx_ar9/ifxmips_3port_sw_reg.h"
#endif /*--- #if defined(CONFIG_AR9) ---*/
#include "cpmac_interface.h"
#include "cpmac_test.h"


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static struct proc_dir_entry* cpmac_proc_dir;


/*------------------------------------------------------------------------------------------*\
 * Displays the version of ETH module via proc file
\*------------------------------------------------------------------------------------------*/
static int cpmac_proc_version(char  *buffer,
                              char **start __attribute__ ((unused)),
                              off_t offset __attribute__ ((unused)),
                              int count __attribute__ ((unused)), 
                              int *eof, 
                              void *data __attribute__ ((unused))) {
    int len = 0;
    /* Ignore the sanity check, because the version fits into a single page */
    len += sprintf(buffer, "CPMAC version: %s\n", AVM_CPMAC_VERSION);
    *eof = 1;
    return len;    
}

/*------------------------------------------------------------------------------------------*\
 * Displays the version of ETH module via proc file
\*------------------------------------------------------------------------------------------*/
static int cpmac_proc_linkstate(char  *buffer,
                                char **start __attribute__ ((unused)),
                                off_t offset __attribute__ ((unused)),
                                int count __attribute__ ((unused)), 
                                int *eof, 
                                void *data __attribute__ ((unused))) {
    int len = 0;
    /* Ignore the sanity check, because the output fits into a single page */
    len += cpmac_interface_print_link_state_output(buffer, PAGE_SIZE - 10);
    *eof = 1;
    return len;    
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
#if defined(CONFIG_AR9)
typedef enum {
    CPMAC_AR9_DUMP_TYPE_SWITCH,
    CPMAC_AR9_DUMP_TYPE_SWITCH_COUNTER,
    CPMAC_AR9_DUMP_TYPE_DONE
} cpmac_ar9_dump_type_t;

static struct {
    volatile cpmac_ar9_dump_type_t type;
    volatile unsigned int          offset;
    volatile unsigned char         port;
} cpmac_ar9_dump_state;


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void *cpmac_proc_reg_dump_ar9_start(struct seq_file *s __attribute__ ((unused)),
                                           loff_t *pos) {
    if(*pos > 0x1b4) {
        return NULL;
    }
    if(*pos != 0) {
        if(cpmac_ar9_dump_state.type == CPMAC_AR9_DUMP_TYPE_DONE) {
            return NULL;
        } else {
            return (void *) &cpmac_ar9_dump_state;
        }
    }
    seq_printf(s, "\nStart dump with offset %#x\n", (unsigned int) *pos);
    cpmac_ar9_dump_state.type   = CPMAC_AR9_DUMP_TYPE_SWITCH;
    cpmac_ar9_dump_state.offset = 0;
    cpmac_ar9_dump_state.port   = 0;
    return (void *) &cpmac_ar9_dump_state;
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void *cpmac_proc_reg_dump_ar9_next(struct seq_file *s __attribute__ ((unused)),
                                          void *v __attribute__ ((unused)),
                                          loff_t *pos __attribute__ ((unused))) {
    switch(cpmac_ar9_dump_state.type) {
        case CPMAC_AR9_DUMP_TYPE_SWITCH:
            cpmac_ar9_dump_state.offset += 4;
            if(cpmac_ar9_dump_state.offset > 0x1b4) {
                cpmac_ar9_dump_state.type   = CPMAC_AR9_DUMP_TYPE_SWITCH_COUNTER;
                cpmac_ar9_dump_state.offset = 0;
                cpmac_ar9_dump_state.port   = 0;
            }
            return (void *) &cpmac_ar9_dump_state;
        case CPMAC_AR9_DUMP_TYPE_SWITCH_COUNTER:
            cpmac_ar9_dump_state.offset++;
            if(cpmac_ar9_dump_state.offset > 0x27) {
                cpmac_ar9_dump_state.offset = 0;
                cpmac_ar9_dump_state.port++;
            }
            if(cpmac_ar9_dump_state.port > 2) {
                cpmac_ar9_dump_state.type = CPMAC_AR9_DUMP_TYPE_DONE;
                cpmac_ar9_dump_state.offset = 0;
                cpmac_ar9_dump_state.port   = 0;
                return NULL;
            }
            return (void *) &cpmac_ar9_dump_state;
        default:
            break;
    }
    return (void *) &cpmac_ar9_dump_state;
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void cpmac_proc_reg_dump_ar9_stop(struct seq_file *s __attribute__ ((unused)),
                                         void *v __attribute__ ((unused))) {
    /* Now erase all counters */
    IFX_REG_W32((1 << 11) | (3 << 9),(volatile unsigned int *) (IFX_AR9_ETH_REG_BASE + 0xc4));
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static int cpmac_proc_reg_dump_ar9_show(struct seq_file *s __attribute__ ((unused)),
                                        void *v __attribute__ ((unused))) {
    unsigned char port;
    static unsigned int counter[3][0x28];

    switch(cpmac_ar9_dump_state.type) {
        case CPMAC_AR9_DUMP_TYPE_SWITCH:
            if(cpmac_ar9_dump_state.offset == 0) {
                seq_printf(s, "Registerdump AR9 Tantos\n");
            }
            seq_printf(s, 
                       "    Register %#05x = %#010x\n", 
                       cpmac_ar9_dump_state.offset,
                       IFX_REG_R32((volatile unsigned int *) (IFX_AR9_ETH_REG_BASE + cpmac_ar9_dump_state.offset)));
            break;
        case CPMAC_AR9_DUMP_TYPE_SWITCH_COUNTER:
            if((cpmac_ar9_dump_state.port == 0) && (cpmac_ar9_dump_state.offset == 0)) {
                seq_printf(s, "\nAR9-Counter\n");
                for(port = 0; port < 3; port++) {
                    unsigned int i;
                    for(i = 0; i <= 0x27; i++) {
                        IFX_REG_W32((1 << 11) | (port << 6) | i, (volatile unsigned int *) (IFX_AR9_ETH_REG_BASE + 0xc4));
                        while((1 << 11) & IFX_REG_R32((volatile unsigned int *) (IFX_AR9_ETH_REG_BASE + 0xc4))) {
                            schedule();
                        }
                        counter[port][i] = IFX_REG_R32((volatile unsigned int *) (IFX_AR9_ETH_REG_BASE + 0xc8));
                    }
                }
            }
            if(cpmac_ar9_dump_state.offset == 0) {
                seq_printf(s, "  Port %u\n", cpmac_ar9_dump_state.port);
            }
            seq_printf(s,
                       "    Register %#04x = %#010x\n", 
                       cpmac_ar9_dump_state.offset,
                       counter[cpmac_ar9_dump_state.port][cpmac_ar9_dump_state.offset]);
            break;
        default:
            break;
    }
    return 0;
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static const struct seq_operations reg_dump_ar9_seq_ops = {
    .start = cpmac_proc_reg_dump_ar9_start,
    .stop  = cpmac_proc_reg_dump_ar9_stop,
    .next  = cpmac_proc_reg_dump_ar9_next,
    .show  = cpmac_proc_reg_dump_ar9_show
};


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static int cpmac_proc_reg_dump_ar9_open(struct inode *inode __attribute__ ((unused)),
                                        struct file *file) {
    return seq_open(file, &reg_dump_ar9_seq_ops);
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static const struct file_operations reg_dump_ar9_proc_ops = {
    .owner   = THIS_MODULE,
    .open    = cpmac_proc_reg_dump_ar9_open,
    .read    = seq_read,
    .llseek  = seq_lseek,
    .release = seq_release
};
#endif /*--- #if defined(CONFIG_AR9) ---*/


#if defined(CONFIG_VR9) & 0
typedef enum {
    CPMAC_VR9_DUMP_TYPE_SWITCH,
    CPMAC_VR9_DUMP_TYPE_SWITCH_COUNTER,
    CPMAC_VR9_DUMP_TYPE_DONE
} cpmac_vr9_dump_type_t;

static struct {
    volatile cpmac_vr9_dump_type_t type;
    volatile unsigned int          offset;
    volatile unsigned char         port;
} cpmac_vr9_dump_state;

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void *cpmac_proc_reg_dump_vr9_start(struct seq_file *s __attribute__ ((unused)),
                                           loff_t *pos) {
    if(*pos > 0x1b4) {
        return NULL;
    }
    if(*pos != 0) {
        if(cpmac_vr9_dump_state.type == CPMAC_VR9_DUMP_TYPE_DONE) {
            return NULL;
        } else {
            return (void *) &cpmac_vr9_dump_state;
        }
    }
    seq_printf(s, "\nStart dump with offset %#x\n", (unsigned int) *pos);
    cpmac_vr9_dump_state.type   = CPMAC_VR9_DUMP_TYPE_SWITCH;
    cpmac_vr9_dump_state.offset = 0;
    cpmac_vr9_dump_state.port   = 0;
    return (void *) &cpmac_vr9_dump_state;
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void *cpmac_proc_reg_dump_vr9_next(struct seq_file *s __attribute__ ((unused)),
                                          void *v __attribute__ ((unused)),
                                          loff_t *pos __attribute__ ((unused))) {
    switch(cpmac_vr9_dump_state.type) {
        case CPMAC_VR9_DUMP_TYPE_SWITCH:
            cpmac_vr9_dump_state.offset += 4;
            if(cpmac_vr9_dump_state.offset > 0x1b4) {
                cpmac_vr9_dump_state.type   = CPMAC_VR9_DUMP_TYPE_SWITCH_COUNTER;
                cpmac_vr9_dump_state.offset = 0;
                cpmac_vr9_dump_state.port   = 0;
            }
            return (void *) &cpmac_vr9_dump_state;
        case CPMAC_VR9_DUMP_TYPE_SWITCH_COUNTER:
            cpmac_vr9_dump_state.offset++;
            if(cpmac_vr9_dump_state.offset > 0x27) {
                cpmac_vr9_dump_state.offset = 0;
                cpmac_vr9_dump_state.port++;
            }
            if(cpmac_vr9_dump_state.port > 2) {
                cpmac_vr9_dump_state.type = CPMAC_VR9_DUMP_TYPE_DONE;
                cpmac_vr9_dump_state.offset = 0;
                cpmac_vr9_dump_state.port   = 0;
                return NULL;
            }
            return (void *) &cpmac_vr9_dump_state;
        default:
            break;
    }
    return (void *) &cpmac_vr9_dump_state;
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void cpmac_proc_reg_dump_vr9_stop(struct seq_file *s __attribute__ ((unused)),
                                         void *v __attribute__ ((unused))) {
    /* Now erase all counters */
    IFX_REG_W32((1 << 11) | (3 << 9),(volatile unsigned int *) (IFX_VR9_ETH_REG_BASE + 0xc4));
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static int cpmac_proc_reg_dump_vr9_show(struct seq_file *s __attribute__ ((unused)),
                                        void *v __attribute__ ((unused))) {
    unsigned char port;
    static unsigned int counter[3][0x28];

    switch(cpmac_vr9_dump_state.type) {
        case CPMAC_VR9_DUMP_TYPE_SWITCH:
            if(cpmac_vr9_dump_state.offset == 0) {
                seq_printf(s, "Registerdump VR9 Tantos\n");
            }
            seq_printf(s, 
                       "    Register %#05x = %#010x\n", 
                       cpmac_vr9_dump_state.offset,
                       IFX_REG_R32((volatile unsigned int *) (IFX_VR9_ETH_REG_BASE + cpmac_vr9_dump_state.offset)));
            break;
        case CPMAC_VR9_DUMP_TYPE_SWITCH_COUNTER:
            if((cpmac_vr9_dump_state.port == 0) && (cpmac_vr9_dump_state.offset == 0)) {
                seq_printf(s, "\nVR9-Counter\n");
                for(port = 0; port < 3; port++) {
                    unsigned int i;
                    for(i = 0; i <= 0x27; i++) {
                        IFX_REG_W32((1 << 11) | (port << 6) | i, (volatile unsigned int *) (IFX_VR9_ETH_REG_BASE + 0xc4));
                        while((1 << 11) & IFX_REG_R32((volatile unsigned int *) (IFX_VR9_ETH_REG_BASE + 0xc4))) {
                            schedule();
                        }
                        counter[port][i] = IFX_REG_R32((volatile unsigned int *) (IFX_VR9_ETH_REG_BASE + 0xc8));
                    }
                }
            }
            if(cpmac_vr9_dump_state.offset == 0) {
                seq_printf(s, "  Port %u\n", cpmac_vr9_dump_state.port);
            }
            seq_printf(s,
                       "    Register %#04x = %#010x\n", 
                       cpmac_vr9_dump_state.offset,
                       counter[cpmac_vr9_dump_state.port][cpmac_vr9_dump_state.offset]);
            break;
        default:
            break;
    }
    return 0;
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static const struct seq_operations reg_dump_vr9_seq_ops = {
    .start = cpmac_proc_reg_dump_vr9_start,
    .stop  = cpmac_proc_reg_dump_vr9_stop,
    .next  = cpmac_proc_reg_dump_vr9_next,
    .show  = cpmac_proc_reg_dump_vr9_show
};


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static int cpmac_proc_reg_dump_vr9_open(struct inode *inode __attribute__ ((unused)),
                                        struct file *file) {
    return seq_open(file, &reg_dump_vr9_seq_ops);
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static const struct file_operations reg_dump_vr9_proc_ops = {
    .owner   = THIS_MODULE,
    .open    = cpmac_proc_reg_dump_vr9_open,
    .read    = seq_read,
    .llseek  = seq_lseek,
    .release = seq_release
};
#endif /*--- #if defined(CONFIG_VR9) & 0 ---*/


#if defined(CONFIG_VR9)
/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static int get_number(char **ptr, int *len) {
    int n, ret = 0;

    /* Skip spaces */
    while(((*ptr)[0] == ' ') && (*len > 0)) {
        (*ptr)++;
        (*len)--;
    }
    while((*len > 0) && (**ptr >= '0') && (**ptr <= '9')) {
        n = **ptr - '0';
        ret = ret * 10 + n;
        (*ptr)++;
        (*len)--;
    }
    return ret;
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
#define AVM_CPMAC_MAX_PROC_INPUT_SIZE 256u
static int cpmac_proc_lan_test_config(struct file  *file __attribute__ ((unused)),
                                      const char   *buf,
                                      unsigned long count,
                                      void         *data __attribute__ ((unused))) {
    char *local_buffer = NULL, *local_buffer_ptr = NULL;
    int len;

    local_buffer = kmalloc(AVM_CPMAC_MAX_PROC_INPUT_SIZE, GFP_KERNEL);
    if(local_buffer == NULL) {
        DEB_ERR("[%s] Could not allocate memory!\n", __FUNCTION__);
        return 0;
    }
    local_buffer_ptr = local_buffer;

    len = min(AVM_CPMAC_MAX_PROC_INPUT_SIZE - 1, (unsigned int) count);
    len = len - copy_from_user(local_buffer, buf, len);
    local_buffer[len] = 0;

    if(strncmp(local_buffer, "init", 4) == 0) {
        unsigned char first, second;
        local_buffer += 4;
        len          -= 4;
        first  = (unsigned char) get_number(&local_buffer, &len);
        second = (unsigned char) get_number(&local_buffer, &len);
        cpmac_test_init(first, second);
    } else if(strncmp(local_buffer, "start", 5) == 0) {
        unsigned int size, no;
        local_buffer += 5;
        len          -= 5;
        size = (unsigned int) get_number(&local_buffer, &len);
        no   = (unsigned int) get_number(&local_buffer, &len);
        cpmac_test_start(size, no);
    } else if(strncmp(local_buffer, "stop", 4) == 0) {
        cpmac_test_deinit();
    } else {
        DEB_WARN("[%s] Could not recognize command. Possible are:\n", __FUNCTION__);
        DEB_WARN("[%s]   init <first port> <second port>\n", __FUNCTION__);
        DEB_WARN("[%s]   start <pkt_size> <number_of_packets>\n", __FUNCTION__);
        DEB_WARN("[%s]   stop\n", __FUNCTION__);
    }
    
    kfree(local_buffer_ptr);

    return (int) count;
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static int cpmac_proc_lan_test_result(char  *buffer,
                                      char **start __attribute__ ((unused)),
                                      off_t offset __attribute__ ((unused)),
                                      int count __attribute__ ((unused)), 
                                      int *eof, 
                                      void *data __attribute__ ((unused))) {
    *eof = 1;
    if(offset > 0) {
        return 0;
    }
    return cpmac_test_lan_result(buffer, count);
}
#endif /*--- #if defined(CONFIG_VR9) ---*/


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
cpmac_err_t cpmac_proc_init(void) {
#   if defined(CONFIG_AR9) || defined(CONFIG_VR9)
    struct proc_dir_entry *entry;
#   endif /*--- #if defined(CONFIG_AR9) || defined(CONFIG_VR9) ---*/

    cpmac_proc_dir = proc_mkdir("driver/avm_cpmac", NULL);
    if(cpmac_proc_dir == NULL) {
        DEB_ERR("[%s] Error! Count not create proc directory /driver/avm_cpmac!\n", __FUNCTION__);
        return CPMAC_ERR_REGISTER_FAILED;
    }
    create_proc_read_entry("version", 0, cpmac_proc_dir, cpmac_proc_version,  NULL);
    create_proc_read_entry("linkstate", 0, cpmac_proc_dir, cpmac_proc_linkstate,  NULL);
#   if defined(CONFIG_AR9) /*--- || (CONFIG_VR9) ---*/
    entry = create_proc_entry("driver/avm_cpmac/reg_dump", 0, NULL);
    if(entry) {
#       if defined(CONFIG_AR9)
        entry->proc_fops = &reg_dump_ar9_proc_ops;
#       elif defined(CONFIG_VR9) & 0 /*--- #if defined(CONFIG_AR9) ---*/
        entry->proc_fops = &reg_dump_vr9_proc_ops;
#       endif /*--- #elif defined(CONFIG_VR9) ---*/ /*--- #if defined(CONFIG_AR9) ---*/
    }
#   endif /*--- #if defined(CONFIG_AR9) || (CONFIG_VR9) ---*/
#   if defined(CONFIG_VR9)
    entry = create_proc_entry("test_lan", 0, cpmac_proc_dir);
    if(entry) {
        entry->read_proc  = cpmac_proc_lan_test_result;
        entry->write_proc = cpmac_proc_lan_test_config;
    }
#   endif /*--- #if defined(CONFIG_VR9) ---*/
    return CPMAC_ERR_NOERR;
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void cpmac_proc_deinit(void) {
    remove_proc_entry("version", cpmac_proc_dir);
    remove_proc_entry("reg_dump",  cpmac_proc_dir);
    remove_proc_entry("test_lan",  cpmac_proc_dir);
    remove_proc_entry("driver/avm_cpmac",  NULL);
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
#if 0
void cpmac_proc_support_data(void) {
#   if defined(CONFIG_MIPS_UR8) || defined(CONFIG_MIPS_OHIO)
#   cpphy_mdio_t *mdio = &cpmac_global.instance[0].mdio; /* FIXME */
    cpphy_cppi_t *cppi = mdio->cpmac_priv->cppi;
    unsigned int i;
    cpphy_tcb_t *tcb;
#   endif /*--- #if defined(CONFIG_MIPS_UR8) || defined(CONFIG_MIPS_OHIO) ---*/

    local_irq_disable();

    reg_dump();

#   if defined(CONFIG_MIPS_UR8) || defined(CONFIG_MIPS_OHIO)
    DEB_SUPPORT("tcb management normal : %u alloced, %u freed\n", 
                cppi->support.tcbs_alloced, 
                cppi->support.tcbs_freed);
    DEB_SUPPORT("tcb management dynamic: %u alloced, %u freed\n", 
                cppi->support.tcbs_alloced_dynamic,
                cppi->support.tcbs_freed_dynamic);
    for(i = 0; i < CPPHY_PRIO_QUEUES; i++) {
        unsigned int count = 0;
        DEB_SUPPORT("tx priority queue %u:\n", i);
        DEB_SUPPORT("    MaxSize       = %u\n", cppi->TxPrioQueues.q[i].MaxSize);
        DEB_SUPPORT("    Free          = %u\n", cppi->TxPrioQueues.q[i].Free);
        for(tcb = (cpphy_tcb_t *) cppi->TxPrioQueues.q[i].First; tcb != NULL; tcb = (cpphy_tcb_t *) tcb->Next, count++)
            ;
        DEB_SUPPORT("      Counted     = %u\n", count);
        DEB_SUPPORT("    MaxDMAFree    = %u\n", cppi->TxPrioQueues.q[i].MaxDMAFree);
        DEB_SUPPORT("    DMAFree       = %u\n", atomic_read(&cppi->TxPrioQueues.q[i].DMAFree));
        DEB_SUPPORT("    Pause         = %u\n", cppi->TxPrioQueues.q[i].Pause);
        DEB_SUPPORT("    PauseInit     = %u\n", cppi->TxPrioQueues.q[i].PauseInit);
        DEB_SUPPORT("    BytesEnqueued = %u\n", cppi->TxPrioQueues.q[i].BytesEnqueued);
        DEB_SUPPORT("    BytesDequeued = %u\n", cppi->TxPrioQueues.q[i].BytesDequeued);
        DEB_SUPPORT("    First         = %p\n", cppi->TxPrioQueues.q[i].First);
        DEB_SUPPORT("    Last          = %p\n", cppi->TxPrioQueues.q[i].Last);
        DEB_SUPPORT("\n");
    }
#   endif /*--- #if defined(CONFIG_MIPS_UR8) || defined(CONFIG_MIPS_OHIO) ---*/
    DEB_SUPPORT("\n");
    
    local_irq_enable();
#   if defined(CONFIG_MIPS_UR8) || defined(CONFIG_MIPS_OHIO)
    tasklet_schedule(&mdio->cpmac_priv->tasklet);
#   endif /*--- #if defined(CONFIG_MIPS_UR8) || defined(CONFIG_MIPS_OHIO) ---*/
}
#endif /*--- #if 0 ---*/


/*------------------------------------------------------------------------------------------*\
 * Dump the given memory location                                                           *
 * Arguments: ptr    - Memory location                                                      *
 *            size   - size of items to output (1, 2, 4 bytes)                              *
 *            length - number of items to output                                            *
\*------------------------------------------------------------------------------------------*/
#if 0
cpmac_err_t cpmac_proc_debug_peek(void *ptr, unsigned int size, unsigned int length) {
#   define PEEK_BUFFER_SIZE 256
    char *buffer;
    unsigned int bufptr, i;

    if(ptr == NULL) {
        DEB_ERR("[%s] NULL pointer!\n", __FUNCTION__);
        return CPMAC_ERR_EXCEEDS_LIMIT;
    }
    buffer = kmalloc(PEEK_BUFFER_SIZE, GFP_KERNEL);
    if(buffer == NULL) {
        DEB_ERR("[%s] Out of memory!\n", __FUNCTION__);
        return CPMAC_ERR_NOMEM;
    }
    bufptr = 0;
    for(i = 1; i <= length; i++) {
        switch(size) {
            case 1:
                bufptr += snprintf(buffer + bufptr, PEEK_BUFFER_SIZE - bufptr, "%02x ", *(((unsigned char *) ptr) + i - 1));
                break;
            case 2:
                bufptr += snprintf(buffer + bufptr, PEEK_BUFFER_SIZE - bufptr, "%04x ", *(((unsigned short *) ptr) + i - 1));
                break;
            case 4:
                bufptr += snprintf(buffer + bufptr, PEEK_BUFFER_SIZE - bufptr, "%08x ", *(((unsigned int *) ptr) + i - 1));
                break;
            default:
                DEB_ERR("[%s] Illegal size (%u) given!\n", __FUNCTION__, size);
                kfree(buffer);
                return CPMAC_ERR_EXCEEDS_LIMIT;
        }
        if(!(i & 0x7)) {
            DEB_SUPPORT("  %s\n", buffer);
            bufptr = 0;
        }
    }
    if(bufptr != 0) {
        DEB_SUPPORT("  %s\n", buffer);
    }
    DEB_SUPPORT("\n");
    kfree(buffer);

    return CPMAC_ERR_NOERR;
#   undef PEEK_BUFFER_SIZE
}
#endif /*--- #if 0 ---*/


/*------------------------------------------------------------------------------------------*\
 * Dump the given memory location                                                           *
 * Arguments: ptr   - Memory location                                                       *
 *            size  - size of items to write (1, 2, 4 bytes)                                *
 *            value - Value to write                                                        *
\*------------------------------------------------------------------------------------------*/
#if 0
cpmac_err_t cpmac_proc_debug_poke(void *ptr, unsigned int size, unsigned int value) {
    if(ptr == NULL) {
        DEB_ERR("[%s] NULL pointer!\n", __FUNCTION__);
        return CPMAC_ERR_EXCEEDS_LIMIT;
    }
    switch(size) {
        case 1:
            *((unsigned char *) ptr) = (unsigned char) value;
            break;
        case 2:
            *((unsigned short *) ptr) = (unsigned short) value;
            break;
        case 4:
            *((unsigned int *) ptr) = value;
            break;
        default:
            DEB_ERR("[%s] Illegal size (%u) given!\n", __FUNCTION__, size);
            return CPMAC_ERR_EXCEEDS_LIMIT;
    }
    return CPMAC_ERR_NOERR;
}
#endif /*--- #if 0 ---*/


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
#if 0
void cpmac_proc_debug(cpphy_mdio_t *mdio __attribute__ ((unused)),
                      unsigned int value __attribute__ ((unused))) {
    /*--- DEB_TEST("[%s] Putting device %u into reset \n", __FUNCTION__, value); ---*/

    /*--- avm_register_reset_gpio(value, HIGH_ACTIVE); ---*/
    /*--- avm_put_device_into_reset(value); ---*/
}
#endif /*--- #if 0 ---*/

