/******************************************************************************
**
** FILE NAME    : ifxmips_3port_eth_sw.c
** PROJECT      : IFX UEIP
** MODULES      : IFX ethenret device driver 
** DATE         : 30 July  2009
** AUTHOR       : Reddy Mallikarjuna 
** DESCRIPTION  : IFX Cross-Platform ethernet device driver 
** COPYRIGHT    :       Copyright (c) 2009
**                      Infineon Technologies AG
**                      Am Campeon 1-12, 85579 Neubiberg, Germany
**
**    This program is free software; you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation; either version 2 of the License, or
**    (at your option) any later version.
**
** HISTORY
** $Date                $Author                 $Comment
** 30 July 2009         Reddy Mallikarjuna      Initial UEIP release
*******************************************************************************/

/*!
  \file ifxmips_3port_eth_sw.c
  \ingroup IFX_ETH_DRV
  \brief ETH driver source file for ARx.
*/

#include <linux/module.h>
#include <linux/kernel.h> /* printk() */
#include <linux/moduleparam.h>
#include <linux/types.h>  /* size_t */
#include <linux/netdevice.h>   /* struct device, and other headers */
#include <linux/ethtool.h>
#include <linux/mii.h>
#include <linux/if_ether.h>
#include <linux/etherdevice.h> /* eth_type_trans */
#include <asm/delay.h>
#include <linux/init.h>
#include <asm/uaccess.h>
#include <linux/proc_fs.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <linux/sched.h>
#include <linux/netdevice.h>

#include <ifx_types.h>
#include <ifx_regs.h>
#include <common_routines.h>
#include <ifx_pmu.h>
#include <ifx_dma_core.h>
#include <linux/workqueue.h>
#include <ifx_eth_framework.h>

#include "ifxmips_eth.h"
#if defined(CONFIG_AR9)
#include "ifxmips_3port_sw_reg.h"
#include "ifxmips_3port_eth_sw.h"
#define RESET_GPHY0                              34  //  P2.2
#define RESET_GPHY1                              36  //  P2.4

#endif /*CONFIG_AR9*/
#include "../cpmac_ar9_if.h"
#include "../cpmac_interface.h"

#define IFX_DRV_MODULE_NAME             "ifxmips_3port_sw"
#define IFX_DRV_MODULE_VERSION          "1.0.5a"
static char version[] = IFX_DRV_MODULE_NAME ".c:v" IFX_DRV_MODULE_VERSION " \n";

/* length of time before we decide the hardware is borked,
 * and dev->tx_timeout() should be called to fix the problem
 */
#define IFX_TX_TIMEOUT                  (10 * HZ)

#define DMA_TX_BURST_LEN                DMA_BURSTL_4DW
#define DMA_RX_BURST_LEN                DMA_BURSTL_4DW
#define ETH_PKT_BUF_SIZE                1568
#define TANTOS_NAPI_WEIGHT              64

#if defined(CONFIG_AR9)
/* Init of the 3 port switch */
int ar9_3port_sw_hw_init(void);
void ar9_3port_sw_hw_uninit(void);
#endif /*--- #if defined(CONFIG_AR9) ---*/

/*------------------------------------------------------------------------------------------*\
 * mii1 gphy status structure
\*------------------------------------------------------------------------------------------*/
static void mii1_gphy_work_handler(struct work_struct *work);
struct workqueue_struct *mii1_gphy_workqueue;
DECLARE_WORK(mii1_gphy_work, mii1_gphy_work_handler);

struct mii1_gphy_status {
    int connected;  /*--- Phy vorhanden ---*/
    int phyaddr;
    unsigned int (*read_phy_int_status)(unsigned int phy_address);
    unsigned int (*read_phy_link_status)(unsigned int phy_address);
    void (*config_phy)(unsigned int phy_address);
    int switchport;
    int link;       /* link status, up/down */
    int speed;      /* preset link speed, 10/100/1000 */
    int duplex;     /* duplex status, FD/HD */
    int flags;      /*--- driver-Flags (10MBit/100MBit/1GBit ---*/
    int eth_up;
    struct net_device *dev;
};
static struct mii1_gphy_status gphy_status[NUM_ETH_INF];

static const int eth_gpio_module_id = IFX_GPIO_MODULE_INTERNAL_SWITCH;

/* pointer to the dma device structure */
struct dma_device_info *g_dma_device=NULL;
/*proc file directory*/
static struct proc_dir_entry*   ifx_eth_dir;
/* Pointer to netword devices structure */
static struct net_device *ifx_eth_dev[NUM_ETH_INF];
/** PMAC header structure */
cpu_egress_pkt_header_t eg_pkt_hdr; 
cpu_ingress_pkt_header_t ig_pkt_hdr;
static int g_pmac_dma, g_dma_pmac;

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static int ifx_eth_open(struct net_device *dev);
static int ifx_eth_close(struct net_device *dev);
/** Transmit packet from Tx Queue to MAC */
static int ifx_start_xmit(struct sk_buff *skb, struct net_device *dev);
/** Hardware specific IOCTL's  */
static int ifx_eth_ioctl(struct net_device *dev, struct ifreq *ifr, int cmd);
/** Get the network device statistics */
static struct net_device_stats *ifx_get_stats (struct net_device *dev);
/** change MTU values */
static int ifx_change_mtu (struct net_device *dev, int new_mtu);
/** Set mac address*/
static int ifx_set_mac_address(struct net_device *dev, void *p); 
/*Ethernet transmit timeout*/
static void ifx_tx_timeout (struct net_device *dev);
/* select the tx dma channel */
static int ifx_select_tx_chan (struct sk_buff *skb, struct net_device *dev);
/*open dma rx channel*/
static void enable_dma_channel(void);  
/*close the dma rx channel*/
static void disable_dma_channel(void); 
/** Init of the network device */
static int switch_init(struct net_device *dev);
/* Get the ether addr from u-boot */
static unsigned char my_ethaddr[MAX_ADDR_LEN];

/* Semaphore, to protect the MDIO accesses */
static struct semaphore mdio_semaphore;

void reset_ext_gphy(int gpio);

/*--- #define PRINT_AR9_DEBUG(args...) printk(args) ---*/
#define PRINT_AR9_DEBUG(args...) // leer

/*--- #define DUMP_PACKET ---*/
#ifdef DUMP_PACKET
/*
* \brief	dump skb data
* \param[in] len length of the data buffer
* \param[in] pData Pointer to data to dump
*
* \return void No Value
*/
inline static void dump_buffer(const char *prefix, char *pData, u32 len){
	int i;
    if(len > 32)
        len = 32;
	printk("[%s] ", prefix);
	for(i=0;i<len;i++){
		printk("%2.2x ",(u8)(pData[i]));
		if (i % 16 == 15)
			printk("\n");
	}
	printk("\n");
}
#endif

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void ar9_switch_write_mdio(unsigned int uPhyAddr, unsigned int uRegAddr,unsigned int data) {
    unsigned int reg;

    if(unlikely(in_atomic())) {
        dump_stack();
        return;
    }
    if(down_interruptible(&mdio_semaphore) != 0) {
        printk(KERN_ERR "[%s] Semaphore interrupted!\n", __FUNCTION__);
        return;
    }

    do {
        schedule();
        reg = IFX_REG_R32(IFX_AR9_ETH_MDIO_CTL);
    } while(reg & MDIO_CTL_MBUSY);

    reg = (MDIO_CTL_WD(data) | MDIO_CTL_OP_WRITE        \
        | MDIO_CTL_MBUSY | MDIO_CTL_PHYAD_SET(uPhyAddr) \
        | MDIO_CTL_REGAD_SET(uRegAddr));
    IFX_REG_W32( reg, IFX_AR9_ETH_MDIO_CTL);

    do {
        schedule();
        reg = IFX_REG_R32(IFX_AR9_ETH_MDIO_CTL);
    } while(reg & MDIO_CTL_MBUSY);

    up(&mdio_semaphore);

    return ; 
}
EXPORT_SYMBOL(ar9_switch_write_mdio);


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
unsigned short ar9_switch_read_mdio(unsigned int uPhyAddr, unsigned int uRegAddr) {
    unsigned int reg;

    if(unlikely(in_atomic())) {
        dump_stack();
        return 0;
    }
    if(down_interruptible(&mdio_semaphore) != 0) {
        printk(KERN_ERR "[%s] Semaphore interrupted!\n", __FUNCTION__);
        return 0;
    }
    do {
        schedule();
        reg = IFX_REG_R32(IFX_AR9_ETH_MDIO_CTL);
    } while(reg & MDIO_CTL_MBUSY);

    reg =   MDIO_CTL_OP_READ | MDIO_CTL_MBUSY \
          | MDIO_CTL_PHYAD_SET(uPhyAddr) | MDIO_CTL_REGAD_SET(uRegAddr);
    IFX_REG_W32( reg, IFX_AR9_ETH_MDIO_CTL);
    do {
        schedule();
        reg = IFX_REG_R32(IFX_AR9_ETH_MDIO_CTL);
    } while(reg & MDIO_CTL_MBUSY);

    up(&mdio_semaphore);
    return(IFX_REG_R32(IFX_AR9_ETH_MDIO_DATA) & MDIO_CTL_RD_MASK);
}
EXPORT_SYMBOL(ar9_switch_read_mdio);


/*------------------------------------------------------------------------------------------*\
 * Get the driver information
\*------------------------------------------------------------------------------------------*/
static void ifx_get_drvinfo(struct net_device *dev __attribute__ ((unused)),
                            struct ethtool_drvinfo *info) {
    /* driver driver short name (Max 32 characters) */
    strcpy (info->driver, IFX_DRV_MODULE_NAME);
     /* driver version (Max 32 characters) */
    strcpy (info->version, IFX_DRV_MODULE_VERSION);
}


/*------------------------------------------------------------------------------------------*\
 * Get the network device settings
 * TODO dev_index
\*------------------------------------------------------------------------------------------*/
static int ifx_get_settings(struct net_device *dev, struct ethtool_cmd *cmd) {
    IFX_switch_priv_t *priv = netdev_priv(dev);
    unsigned int  port_status;
    
    int dev_index = !(strcmp(dev->name, "eth0")) ? gphy_status[0].switchport : gphy_status[1].switchport;

    PRINT_AR9_DEBUG("[%s] device %s\n", __FUNCTION__, dev->name);
    cmd->supported = (SUPPORTED_Autoneg | SUPPORTED_TP);

#if 0
    cmd->supported |= (SUPPORTED_100baseT_Half |    \
                       SUPPORTED_100baseT_Full |       \
                       SUPPORTED_10baseT_Half |        \
                       SUPPORTED_10baseT_Full |        \
                       /*--- SUPPORTED_1000baseT_Half |      \ ---*/
                       /*--- SUPPORTED_1000baseT_Full |      \ ---*/
                       SUPPORTED_MII);    
#else
    cmd->supported |= gphy_status[dev_index].flags | SUPPORTED_MII;
#endif
    
    cmd->advertising = 0;
    if (priv->flags & IFX_FLAG_ADV_10HALF)
        cmd->advertising |= ADVERTISED_10baseT_Half;
    if (priv->flags & IFX_FLAG_ADV_10FULL)
        cmd->advertising |= ADVERTISED_10baseT_Full;
    if (priv->flags & IFX_FLAG_ADV_100HALF)
        cmd->advertising |= ADVERTISED_100baseT_Half;
    if (priv->flags & IFX_FLAG_ADV_100FULL)
        cmd->advertising |= ADVERTISED_100baseT_Full;
    if (priv->flags & IFX_FLAG_ADV_1000HALF)
        cmd->advertising |= ADVERTISED_1000baseT_Half;
    if (priv->flags & IFX_FLAG_ADV_1000FULL)
        cmd->advertising |= ADVERTISED_1000baseT_Full;  
        
    cmd->advertising |= ADVERTISED_Pause | ADVERTISED_Asym_Pause;
    
    port_status = IFX_REG_R32(IFX_AR9_ETH_PS);
    switch( (port_status>> (1+(dev_index*0x8)) ) & 0x3) {
        case 0:
            priv->current_speed=SPEED_10;
            break;
        case 1:
            priv->current_speed=SPEED_100;
            break;
        case 2:
            priv->current_speed=SPEED_1000;
            break;
        default: /* should not be here */
            printk(KERN_ERR "%s[%d]: Port status error(%d)!!!\n", __func__,__LINE__,port_status);
    }
    cmd->speed = priv->current_speed;
    if ( (port_status  >> (3+(dev_index *0x8))) & 0x1)
        priv->full_duplex=DUPLEX_FULL;
    else
        priv->full_duplex=DUPLEX_HALF;
    cmd->duplex = priv->full_duplex;
    
    if ( (port_status >> (8*dev_index)) & 0x1 )
        cmd->reserved[0]=0x1;  /*link up bit:0*/
    else
        cmd->reserved[0]=0x0;  /*link down */
    if (port_status & 0x1000 )
        cmd->reserved[0] |=0x2; /*flow control enable Bit1:1*/
    else
        cmd->reserved[0] |= 0x0; /*flow control disable*/
#if 0
    if (dev_index)
        cmd->phy_address = gphy_status[dev_index].phyaddr;
    else
        cmd->phy_address = PHY0_ADDR;
#else
    cmd->phy_address = gphy_status[dev_index].phyaddr;
#endif
    cmd->port = PORT_MII;
    
    cmd->transceiver = (priv->flags & IFX_FLAG_INTERNAL_PHY) ? XCVR_INTERNAL : XCVR_EXTERNAL;
    return 0;
}


/*------------------------------------------------------------------------------------------*\
 * Set the network device settings
\*------------------------------------------------------------------------------------------*/
static int ifx_set_settings(struct net_device *dev, struct ethtool_cmd *cmd) {
    IFX_switch_priv_t *priv = netdev_priv(dev);
    unsigned int  reg;

    int dev_index = !(strcmp(dev->name, "eth0")) ? gphy_status[0].switchport : gphy_status[1].switchport;
    
    PRINT_AR9_DEBUG("[%s] device %s\n", __FUNCTION__, dev->name);
    if (cmd->autoneg == AUTONEG_ENABLE) {
        /*TODO*/
        return 0;
    } else if ((cmd->speed != SPEED_100 && cmd->speed != SPEED_10 && cmd->speed != SPEED_1000) \
            ||(cmd->duplex != DUPLEX_HALF && cmd->duplex != DUPLEX_FULL)) {
            return -EINVAL;
    }
    spin_lock_irq(&priv->lock);
    if (cmd->autoneg != AUTONEG_ENABLE) {
        reg = IFX_REG_R32(IFX_AR9_ETH_RGMII_CTL);
        reg &= ~RGMII_CTL_SPD_MASK(dev_index);
        reg &= ~RGMII_CTL_DUP_MASK(dev_index);
        priv->flags |= IFX_FLAG_FORCE_LINK;
        priv->flags &= ~(IFX_FLAG_100_BASE_T | IFX_FLAG_1000_BASE_T | IFX_FLAG_FULL_DUPLEX);
        if (cmd->speed == SPEED_100) {
            priv->flags |= IFX_FLAG_100_BASE_T;
            reg |= RGMII_CTL_SPD_100_SET(dev_index);
        } else if(cmd->speed == SPEED_1000) {
            reg |= RGMII_CTL_SPD_1000_SET(dev_index);
            priv->flags |= IFX_FLAG_1000_BASE_T;
        } 
        if (cmd->duplex == DUPLEX_FULL) {
            priv->flags |= IFX_FLAG_FULL_DUPLEX;
            reg |= RGMII_CTL_DUP_SET(dev_index, 1);
        }
        IFX_REG_W32(reg, IFX_AR9_ETH_RGMII_CTL);
        if (dev_index) {
            reg = IFX_REG_R32(IFX_AR9_ETH_P1_CTL);
            reg |= (PX_CTL_FLP | PX_CTL_BYPASS);
            IFX_REG_W32(reg, IFX_AR9_ETH_P1_CTL);
        } else {
            reg = IFX_REG_R32(IFX_AR9_ETH_P0_CTL);
            reg |= (PX_CTL_FLP | PX_CTL_BYPASS);
            IFX_REG_W32(reg, IFX_AR9_ETH_P0_CTL);
        }
    }
    spin_unlock_irq(&priv->lock);
    return 0;
}

/*------------------------------------------------------------------------------------------*\
 * Reset the device
 * Aus sicht "von oben" gibt es nur das device "cpmac0"
\*------------------------------------------------------------------------------------------*/
static int ifx_nway_reset(struct net_device *dev __attribute__ ((unused))) {
    reset_ext_gphy(GPIO_RESET_GPHY0);
#if defined( CONFIG_IFX_MAC1) && CONFIG_IFX_MAC1
    reset_ext_gphy(GPIO_RESET_GPHY1);
#endif
    return 0;
}


/*------------------------------------------------------------------------------------------*\
 * Structure of the ether tool operation
\*------------------------------------------------------------------------------------------*/
static const struct ethtool_ops ifx_ethtool_ops = {
        .get_drvinfo            = ifx_get_drvinfo,
        .get_settings           = ifx_get_settings,
        .set_settings           = ifx_set_settings,
        .nway_reset             = ifx_nway_reset,
        .get_link               = ethtool_op_get_link,

};

/*------------------------------------------------------------------------------------------*\
 * Get the ether addr from u-boot
\*------------------------------------------------------------------------------------------*/
static int __init ethaddr_setup(char *line) {
    char *ep;
    int i;
    memset(my_ethaddr, 0, MAX_ADDR_LEN);
    /* there should really be routines to do this stuff */
    for (i = 0; i < 6; i++)	{
        my_ethaddr[i] = line ? simple_strtoul(line, &ep, 16) : 0;
        if (line)
            line = (*ep) ? ep+1 : ep;
    }
    printk("mac address %2x-%2x-%2x-%2x-%2x-%2x \n" \
        ,my_ethaddr[0]  \
        ,my_ethaddr[1]  \
        ,my_ethaddr[2]  \
        ,my_ethaddr[3]  \
        ,my_ethaddr[4]  \
        ,my_ethaddr[5]);
    return 0;
}
__setup("ethaddr=", ethaddr_setup);

/*------------------------------------------------------------------------------------------*\
 * Turn On RX DMA channels
\*------------------------------------------------------------------------------------------*/
int dma_open_level = 0;
static void enable_dma_channel(void) {
    struct dma_device_info* dma_dev=g_dma_device;
    int i;
    if(dma_open_level) {
        printk(KERN_ERR "[%s] dma already enabled (called by %pF)\n", __FUNCTION__, __builtin_return_address(1));
        return;
    }
    dma_open_level++;
    
    for(i = 0 ; i < dma_dev->max_rx_chan_num ; i++) {
        if ( (dma_dev->rx_chan[i])->control == IFX_DMA_CH_ON )
            dma_dev->rx_chan[i]->open(dma_dev->rx_chan[i]);
    }
}


/*------------------------------------------------------------------------------------------*\
 * Turn Off RX DMA channels
\*------------------------------------------------------------------------------------------*/
static void disable_dma_channel() {
    struct dma_device_info* dma_dev=g_dma_device;
    int i;
    if(dma_open_level != 1) {
        printk(KERN_ERR "[%s] dma already disabled (called by %pF)\n", __FUNCTION__, __builtin_return_address(1));
        return;
    }
    dma_open_level = 0;
    for (i=0; i<dma_dev->max_rx_chan_num; i++)
        dma_dev->rx_chan[i]->close(dma_dev->rx_chan[i]);
}

/*------------------------------------------------------------------------------------------*\
 * open the network device interface
\*------------------------------------------------------------------------------------------*/
static int ifx_eth_open(struct net_device *dev) {
    IFX_switch_priv_t *priv;
    unsigned int reg_val;
    signed int port = -1;

    priv = netdev_priv(dev);
    if(strcmp(dev->name, "eth0") == 0) {
        port = 0;
        priv->port = 0;
    }
    if(strcmp(dev->name, "eth1") == 0) {
        port = 1;
        priv->port = 1;
    }
    if(port < 0) {
        printk(KERN_ERR "[%s] unknown device '%s'\n", __FUNCTION__, dev->name);
        return -ENXIO;
    }

    cpmac_ar9_open(dev);

    switch(gphy_status[port].switchport) {
        case 0:
            reg_val  = IFX_REG_R32(IFX_AR9_ETH_P0_CTL);
            reg_val &= ~(PX_CTL_FLD);
            reg_val |= PX_CTL_FLP;
            IFX_REG_W32(reg_val, IFX_AR9_ETH_P0_CTL);
            break;
        case 1:
            reg_val  = IFX_REG_R32(IFX_AR9_ETH_P1_CTL);
            reg_val &= ~(PX_CTL_FLD);
            reg_val |= PX_CTL_FLP;
            IFX_REG_W32(reg_val, IFX_AR9_ETH_P1_CTL);
            break;
        default:
            printk(KERN_ERR "[%s] unknown phydevice '%d'\n", __FUNCTION__, gphy_status[port].switchport);
    }

    gphy_status[port].dev = dev;
    if (netif_carrier_ok(dev)) {
        netif_start_queue(dev);
        napi_enable(&priv->napi);
    } else {
        netif_stop_queue(dev);
        napi_disable(&priv->napi);
    } 
    gphy_status[port].eth_up = 1;

    enable_dma_channel();
    return IFX_SUCCESS;
}

/*------------------------------------------------------------------------------------------*\
 * Close the network device interface
\*------------------------------------------------------------------------------------------*/
static int ifx_eth_close(struct net_device *dev) {
    IFX_switch_priv_t *priv;
    unsigned int reg_val;
    int port = -1;

    if(strcmp(dev->name, "eth0") == 0) {
        port = 0;
    }
    if(strcmp(dev->name, "eth1") == 0) {
        port = 1;
    }

    disable_dma_channel();

    if((port < 0) || (port > 1)) {
        printk(KERN_ERR "[%s] unknown device '%s'\n", __FUNCTION__, dev->name);
    } else {
        switch (gphy_status[port].switchport) {
            case 0:
                reg_val = IFX_REG_R32(IFX_AR9_ETH_P0_CTL);
                reg_val &= ~(PX_CTL_FLP);
                reg_val |= PX_CTL_FLD;
                IFX_REG_W32(reg_val, IFX_AR9_ETH_P0_CTL);
                gphy_status[0].eth_up = 0;
                gphy_status[0].dev = NULL;
                break;
            case 1:
                reg_val = IFX_REG_R32(IFX_AR9_ETH_P1_CTL);
                reg_val &= ~(PX_CTL_FLP);
                reg_val |= PX_CTL_FLD;
                IFX_REG_W32(reg_val, IFX_AR9_ETH_P1_CTL);
                gphy_status[1].eth_up = 0;
                gphy_status[1].dev = NULL;
                break;
            default:
                printk(KERN_ERR "[%s] unknown phydevice '%d'\n", __FUNCTION__, gphy_status[port].switchport);
        }
    }
    netif_stop_queue(dev);
	priv = netdev_priv(dev);
    napi_disable(&priv->napi);
    return IFX_SUCCESS;
}


/*------------------------------------------------------------------------------------------*\
 * Send the packet to netwrok rx queue, used by  switch_hw_receive function
\*------------------------------------------------------------------------------------------*/
static void ifx_eth_rx(struct net_device *dev, int len,struct sk_buff* skb) {
    IFX_switch_priv_t *priv = netdev_priv(dev);

    /*--- PRINT_AR9_DEBUG(KERN_ERR "[%s] %s\n", __FUNCTION__, dev->name); ---*/
    /*--- PRINT_AR9_DEBUG(KERN_ERR "[%s] %#*B\n", __FUNCTION__, len < 128 ? len : 128, skb->data); ---*/

    skb->dev = dev;
    skb->protocol = eth_type_trans(skb, dev);
    priv->stats.rx_packets++;
    priv->stats.rx_bytes += len;
    netif_receive_skb(skb);
}

/*------------------------------------------------------------------------------------------*\
* This function is called in dma intr handler (DMA RCV_INT interrupt).
* This function get the data from the DMA device. 
*   if the packet is valid then it will send to upper layer based on  criteria.
*       The switch CPU port PMAC status header is enabled, then remove the header and 
*           look from which port the packet comes and send to relative network device.
        if PMAC status header is not enabled, then send the packets eth0 interafce
\*------------------------------------------------------------------------------------------*/
static void switch_hw_receive(struct dma_device_info* dma_dev) {
    unsigned char* buf=NULL;
    int len=0 ;
    struct sk_buff *skb=NULL;
    struct net_device *dev;

    len = dma_device_read(dma_dev, &buf, (void**)&skb);
#ifdef DUMP_PACKET
    dump_buffer(__FUNCTION__,skb->data, len);
#endif
    if ((len >= 0x600) || (len < 64) ) {
        printk(KERN_ERR "%s[%d]: Packet is too large/small (%d)!!!\n", __func__,__LINE__,len);
        goto rx_err_exit;
    }
    if (skb == NULL  ) {
        printk(KERN_ERR "%s[%d]: Can't restore the Packet !!!\n", __func__,__LINE__);
        goto rx_err_exit;
    }
    /* remove CRC */
    len -= 4;
    if (len > (skb->end -skb->tail)) {
        printk(KERN_ERR "%s[%d]: len:%d end:%p tail:%p Err!!!\n", __func__,__LINE__,(len+4), skb->end, skb->tail);
        goto rx_err_exit;
    }
    skb_put(skb, len);
    if(g_pmac_dma){
        int sourcePortId;;
        len -= 8;  /*Remove PMAC to DMA header */
        skb_pull(skb,8);
        eg_pkt_hdr = * ((cpu_egress_pkt_header_t *) (buf + 2));
        sourcePortId =(eg_pkt_hdr.srcPortID) & 0x7 ;
        if(sourcePortId < NUM_ETH_INF) {
            switch(sourcePortId) {
                case 0:
                    dev = ifx_eth_dev[0];
                    break;
                case 1:
                    dev = ifx_eth_dev[1];
                    break;
#               if (NUM_ETH_INF > 2)
                case 2:
                    dev = ifx_eth_dev[2];
                    break;
#               if (NUM_ETH_INF > 3)
                case 3:
                    dev = ifx_eth_dev[3];
                    break;
#               if (NUM_ETH_INF > 4)
                case 4:
                    dev = ifx_eth_dev[4];
                    break;
#               if (NUM_ETH_INF > 5)
                case 5:
                    dev = ifx_eth_dev[5];
                    break;
#               endif /*--- #if (NUM_ETH_INF > 5) ---*/
#               endif /*--- #if (NUM_ETH_INF > 4) ---*/
#               endif /*--- #if (NUM_ETH_INF > 3) ---*/
#               endif /*--- #if (NUM_ETH_INF > 2) ---*/
                default:
                goto rx_err_exit;
            }
            /*--- PRINT_AR9_DEBUG(KERN_ERR "[%s] sourcePortId %d name '%s'\n", __FUNCTION__, sourcePortId, dev->name); ---*/
        } else {
            /*--- PRINT_AR9_DEBUG(KERN_ERR "[%s] sourcePortId %d name '%s'  ERROR!\n", __FUNCTION__, sourcePortId, dev->name); ---*/
            goto rx_err_exit;
        }
    } else {
        dev= ifx_eth_dev[0];
        printk(KERN_ERR "[%s] (no_pmac_dma) name '%s'\n", __FUNCTION__, dev->name);
    }
	    
#   ifdef CONFIG_AVM_PA
    if(avm_pa_dev_receive(AVM_PA_DEVINFO(dev), skb) == 0)
       return;
#   endif /*--- #ifdef CONFIG_AVM_PA ----*/
    skb->dev = dev;
    ifx_eth_rx(dev, len, skb);
    return ;
    
rx_err_exit:
    if (skb)
        dev_kfree_skb_any(skb);
    return ;
}


/*------------------------------------------------------------------------------------------*\
 * Get the network device stats information
\*------------------------------------------------------------------------------------------*/
static struct net_device_stats *ifx_get_stats (struct net_device *dev) {
    IFX_switch_priv_t *priv = netdev_priv(dev);
    return &priv->stats;
}


/*------------------------------------------------------------------------------------------*\
 * Trasmit timeout
\*------------------------------------------------------------------------------------------*/
static void ifx_tx_timeout(struct net_device *dev) {
    IFX_switch_priv_t *priv		= netdev_priv(dev);
    struct dma_device_info* dma_dev = g_dma_device;
    int i;

    PRINT_AR9_DEBUG(KERN_ERR "%s: transmit timed out, disable the dma channel irq\n", dev->name);
    priv->stats.tx_errors++;
    for (i=0; i<dma_dev->num_tx_chan; i++) {
        dma_dev->tx_chan[i]->disable_irq(dma_dev->tx_chan[i]);
    }
    netif_wake_queue(dev);
}


/*------------------------------------------------------------------------------------------*\
 * set mac address, used by ifx_set_mac_address function
\*------------------------------------------------------------------------------------------*/
#if 0
static void __ifx_set_mac_addr (unsigned char *data)
{
    u32 val;
    val  = ((u32) data[2]) << 0;
    val |= ((u32) data[3]) << 8;
    val |= ((u32) data[4]) << 16;
    val |= ((u32) data[5]) << 24;
    IFX_REG_W32(val, IFX_AR9_ETH_PMAC_SA2);
    val  = ((u32) data[0]) << 16;
    val |= ((u32) data[1]) << 24;
    IFX_REG_W32(val, IFX_AR9_ETH_PMAC_SA1);
}
#endif


/*------------------------------------------------------------------------------------------*\
 * Set the MAC address
\*------------------------------------------------------------------------------------------*/
static int ifx_set_mac_address (struct net_device *dev, void *p) {
/*
    IFX_switch_priv_t *priv		= netdev_priv(dev);
*/

    struct sockaddr *addr = p;

    if (netif_running(dev))
        return -EBUSY;

    if (!is_valid_ether_addr(addr->sa_data))
        return -EINVAL;

    memcpy(dev->dev_addr, addr->sa_data, dev->addr_len);
#if 0
    /** TODO: is it required to set MAC address in the HW registers?*/
    spin_lock_irq(&priv->lock);
    if (!(dev->flags & IFF_PROMISC)) {
        __ifx_set_mac_addr(dev->dev_addr);
    }
    spin_unlock_irq(&priv->lock);
#endif
    return 0;
}


/*------------------------------------------------------------------------------------------*\
 * Change the MTU value of the netwrok device interfaces
\*------------------------------------------------------------------------------------------*/
static int ifx_change_mtu (struct net_device *dev, int new_mtu) {
    if(new_mtu < ETH_ZLEN || new_mtu > ETH_DATA_LEN)
        return -EINVAL;
    dev->mtu = new_mtu;
    return 0;
}


/*------------------------------------------------------------------------------------------*\
 * Select the DMA channel numbers, refer dma_setup_init function
\*------------------------------------------------------------------------------------------*/
static int ifx_select_tx_chan(struct sk_buff *skb __attribute__ ((unused)),
                              struct net_device *dev) {
    IFX_switch_priv_t *priv = netdev_priv(dev);
    return gphy_status[priv->port].switchport;
    /*--- return !strcmp(dev->name, "eth0") ? gphy_status[0].switchport : gphy_status[1].switchport; ---*/
}

/*------------------------------------------------------------------------------------------*\
 * Transmit packet over DMA, which comes from the Tx Queue 
 * Note: Tx packet based on the interface queue.
 *       if packet comes from eth0, then sendout the packet over Tx DMA channel 0
 *       if packet comes from eth1, then sendout the packet over Tx DMA channel 1
 * refer the function "ifx_select_tx_chan" selection of dma channels
 * if switch CPU port PMAC status header is enabled, then set the status header 
 *   based on criteria and push the status header infront of header.
 * if head room is not availabe for status header(4-bytes), reallocate the head room 
 *   and push status header  infront of the header
\*------------------------------------------------------------------------------------------*/
static int ifx_start_xmit(struct sk_buff *skb, struct net_device *dev) {
    IFX_switch_priv_t *priv		= netdev_priv(dev);
    int len , rc = NETDEV_TX_OK;
    char *data;
    struct dma_device_info *dma_dev = g_dma_device;

    if (skb ==  NULL) {
        printk(KERN_ERR "%s[%d]: Empty skb given!\n", __func__,__LINE__);
        return -EFAULT;
    }
    len = skb->len < ETH_ZLEN ? ETH_ZLEN : skb->len;
#ifdef DUMP_PACKET
    dump_buffer(__FUNCTION__,skb->data, len);
#endif
    
    if(g_dma_pmac) {
        memset((void *) &ig_pkt_hdr, 0, sizeof (ig_pkt_hdr));
        /*if DIRECT is set 1, then packet is directly forward to the physical port based on DPID (no learning),
		if DIRECT is set 0, then switch will learn and decide forward the packet to which physical port*/
		
#ifdef CONFIG_SW_ROUTING_MODE
        ig_pkt_hdr.DIRECT= 1;
#else
        ig_pkt_hdr.DIRECT= 0;
#endif
        ig_pkt_hdr.SPID = 2;  /* CPU port */
        switch(priv->port) {
            case 0:
                ig_pkt_hdr.DPID = gphy_status[0].switchport; /* Packet send through MIIx interface, valid only when DIRECT is set 1 */
                break;
            case 1:
                ig_pkt_hdr.DPID = gphy_status[1].switchport; /* Send packet through  MIIx interface, valid only when DIRECT is set 1 */
                break;
            default:
                PRINT_AR9_DEBUG(KERN_ERR "[%s] file=%s line=%d name=%s (not eth0 or eth1) ignore\n", __FUNCTION__, __FILE__, __LINE__, dev->name);
                priv->stats.tx_errors++;
                priv->stats.tx_dropped++;
                dev_kfree_skb_any(skb);
                return -ENXIO;
        }
        if(skb_headroom(skb) >= 4) {
            skb_push(skb, 4);
            memcpy(skb->data, (void*)&ig_pkt_hdr, 4);
            len += 4;
        } else {
            struct sk_buff *tmp = skb;
            skb = skb_realloc_headroom(tmp, 4);
            if(tmp)
                dev_kfree_skb_any(tmp);
            if (skb ==  NULL) {
                printk(KERN_ERR "%s[%d]: skb_realloc_headroom failed!!!\n", __func__,__LINE__);
                return -ENOMEM;
            }
            skb_push(skb, 4);
            memcpy(skb->data, &ig_pkt_hdr, 4);
            len += 4;
        }
    } 
    
    data = skb->data;
    priv->skb = skb;
    dev->trans_start = jiffies;
    /*select the tx channel*/
    dma_dev->current_tx_chan = ifx_select_tx_chan (skb, dev);

    if (dma_device_write(dma_dev, data, len, skb) != len ) {
        dev_kfree_skb_any(skb);
        priv->stats.tx_errors++;
        priv->stats.tx_dropped++;
    } else {
        priv->stats.tx_packets++;
        priv->stats.tx_bytes+=len;
    }
    return rc;
}


/*------------------------------------------------------------------------------------------*\
 * Enable VLAN Class oF Service
\*------------------------------------------------------------------------------------------*/
static void set_vlan_cos(_vlan_cos_req* vlan_cos_req) {
    unsigned int pri = vlan_cos_req->pri;
    unsigned int cos_value = vlan_cos_req->cos_value;
    unsigned int reg_val = IFX_REG_R32(IFX_AR9_ETH_1P_PRT);
    reg_val &= ~(3 << (pri * 2));
    reg_val |= ( cos_value << (pri * 2) );
    IFX_REG_W32(reg_val, IFX_AR9_ETH_1P_PRT);
}


/*------------------------------------------------------------------------------------------*\
 * Enable DSCP Class Of Service
\*------------------------------------------------------------------------------------------*/
static void set_dscp_cos(_dscp_cos_req* dscp_cos_req) {
    int dscp = dscp_cos_req->dscp;
    int cos_value = dscp_cos_req->cos_value;
    unsigned int reg_val = IFX_REG_R32(IFX_AR9_ETH_DFSRV_MAP0 +(dscp/16*4));
    reg_val &= ~(3 << (dscp%16*2));
    reg_val |= (cos_value << (dscp%16*2));
    IFX_REG_W32(reg_val, (IFX_AR9_ETH_DFSRV_MAP0 + (dscp/16*4)));
}

/*------------------------------------------------------------------------------------------*\
 * Platform specific IOCTL's handler
\*------------------------------------------------------------------------------------------*/
static int ifx_eth_ioctl(struct net_device *dev, struct ifreq *ifr, int cmd) {
  /*  IFX_switch_priv_t *priv = netdev_priv(dev); */
    _vlan_cos_req etop_vlan_cos_req;
    _dscp_cos_req etop_dscp_cos_req;
    unsigned int reg_val;
    int err = -EINVAL;

    int dev_index = !(strcmp(dev->name, "eth0")) ? gphy_status[0].switchport : gphy_status[1].switchport;

    printk("{%s}\n %s", __FUNCTION__, dev->name);

    if (!netif_running(dev))
        goto out;
    err = 0;
    switch(cmd){
        case SET_VLAN_COS:
            copy_from_user(&etop_vlan_cos_req, (_vlan_cos_req*)ifr->ifr_data,sizeof(_vlan_cos_req));
            set_vlan_cos(&etop_vlan_cos_req);
            break;
        case SET_DSCP_COS:
            copy_from_user(&etop_dscp_cos_req, (_vlan_cos_req*)ifr->ifr_data,sizeof(_dscp_cos_req));
            set_dscp_cos(&etop_dscp_cos_req);
            break;
        case ENABLE_VLAN_COS:
			if (dev_index) {
			    reg_val = IFX_REG_R32(IFX_AR9_ETH_P1_CTL);
			    reg_val |= PX_CTL_VPE;
			    IFX_REG_W32(reg_val, IFX_AR9_ETH_P1_CTL);
			} else {
			    reg_val = IFX_REG_R32(IFX_AR9_ETH_P0_CTL);
			    reg_val |= PX_CTL_VPE;
			    IFX_REG_W32(reg_val, IFX_AR9_ETH_P0_CTL);
			}
            break;
        case DISABLE_VLAN_COS:
            if (dev_index) {
			    reg_val = IFX_REG_R32(IFX_AR9_ETH_P1_CTL);
			    reg_val &= ~PX_CTL_VPE;
			    IFX_REG_W32(reg_val, IFX_AR9_ETH_P1_CTL);
			} else {
			    reg_val = IFX_REG_R32(IFX_AR9_ETH_P0_CTL);
			    reg_val &= ~PX_CTL_VPE;
			    IFX_REG_W32(reg_val, IFX_AR9_ETH_P0_CTL);
			}
            break;
            
        case ENABLE_DSCP_COS:
            if (dev_index) {
			    reg_val = IFX_REG_R32(IFX_AR9_ETH_P1_CTL);
			    reg_val |= PX_CTL_SPE;
			    IFX_REG_W32(reg_val, IFX_AR9_ETH_P1_CTL);
			} else {
			    reg_val = IFX_REG_R32(IFX_AR9_ETH_P0_CTL);
			    reg_val |= PX_CTL_SPE;
			    IFX_REG_W32(reg_val, IFX_AR9_ETH_P0_CTL);
			}
            break;
        case DISABLE_DSCP_COS:
           if (dev_index) {
			    reg_val = IFX_REG_R32(IFX_AR9_ETH_P1_CTL);
			    reg_val &= ~PX_CTL_SPE;
			    IFX_REG_W32(reg_val, IFX_AR9_ETH_P1_CTL);
			} else {
			    reg_val = IFX_REG_R32(IFX_AR9_ETH_P0_CTL);
			    reg_val &= ~PX_CTL_SPE;
			    IFX_REG_W32(reg_val, IFX_AR9_ETH_P0_CTL);
			}
            break;
        case PASS_UNICAST_PACKETS:
            reg_val = IFX_REG_R32(IFX_AR9_ETH_DF_PORTMAP_REG);
            reg_val |= DF_PORTMAP_UP(dev_index);
            IFX_REG_W32(reg_val, IFX_AR9_ETH_DF_PORTMAP_REG);
            break;
        case FILTER_UNICAST_PACKETS:
            reg_val = IFX_REG_R32(IFX_AR9_ETH_DF_PORTMAP_REG);
            reg_val &= ~DF_PORTMAP_UP(dev_index);
            IFX_REG_W32(reg_val, IFX_AR9_ETH_DF_PORTMAP_REG);
            break;
        case KEEP_BROADCAST_PACKETS:
            reg_val = IFX_REG_R32(IFX_AR9_ETH_DF_PORTMAP_REG);
            reg_val |= DF_PORTMAP_BP(dev_index);
            IFX_REG_W32(reg_val, IFX_AR9_ETH_DF_PORTMAP_REG);
            break;
        case DROP_BROADCAST_PACKETS:
            reg_val = IFX_REG_R32(IFX_AR9_ETH_DF_PORTMAP_REG);
            reg_val &= ~DF_PORTMAP_BP(dev_index);
            IFX_REG_W32(reg_val, IFX_AR9_ETH_DF_PORTMAP_REG);
            break;
        case KEEP_MULTICAST_PACKETS:
            reg_val = IFX_REG_R32(IFX_AR9_ETH_DF_PORTMAP_REG);
            reg_val |= DF_PORTMAP_MP(dev_index);
            IFX_REG_W32(reg_val, IFX_AR9_ETH_DF_PORTMAP_REG);
            break;
        case DROP_MULTICAST_PACKETS:
            reg_val = IFX_REG_R32(IFX_AR9_ETH_DF_PORTMAP_REG);
            reg_val &= ~DF_PORTMAP_MP(dev_index);
            IFX_REG_W32(reg_val, IFX_AR9_ETH_DF_PORTMAP_REG);
            break;
#       if defined(CONFIG_AVM_CPMAC)
        case AVM_CPMAC_IOCTL_GENERIC:
            if(!IS_ERR(&cpmac_ar9_ioctl) && &cpmac_ar9_ioctl) {
                err = cpmac_ar9_ioctl(dev, ifr, cmd);
            }
            break;
#       endif /*--- #if defined(CONFIG_AVM_CPMAC) ---*/
        default:
            err = -EINVAL;
            break;
    }
out:
    return err;
}

/*------------------------------------------------------------------------------------------*\
 * DMA Pseudo Interrupt handler.
 * This function handle the DMA pseudo interrupts to handle the data packets Rx/Tx over DMA channels
 * It will handle the following interrupts
 *   RCV_INT: DMA receive the packet interrupt,So get from the PPE peripheral
 *   TX_BUF_FULL_INT: TX channel descriptors are not availabe, so, stop the transmission 
         and enable the Tx channel interrupt.
 *   TRANSMIT_CPT_INT: Tx channel descriptors are availabe and resume the transmission and 
         disable the Tx channel interrupt.
\*------------------------------------------------------------------------------------------*/
int sw_dma_intr_handler(struct dma_device_info* dma_dev, int status) {
    struct net_device* dev;
    int i;
    
    switch(status) {
        case RCV_INT:
            switch_hw_receive(dma_dev);
            break;
        case TX_BUF_FULL_INT:
            for(i=0; i < NUM_ETH_INF ; i++) {
                if (!gphy_status[i].connected)
                    continue;
                dev = ifx_eth_dev[i];
                netif_stop_queue(dev);
            }
            for(i=0;i<dma_dev->num_tx_chan;i++) {
                if((dma_dev->tx_chan[i])->control==IFX_DMA_CH_ON)
                    dma_dev->tx_chan[i]->enable_irq(dma_dev->tx_chan[i]);
            }
            break;
        case TRANSMIT_CPT_INT:
            for(i=0;i<dma_dev->num_tx_chan;i++) {
                dma_dev->tx_chan[i]->disable_irq(dma_dev->tx_chan[i]);
            }
            for(i=0; i < NUM_ETH_INF ; i++) {
                if (!gphy_status[i].connected)
                    continue;
                dev = ifx_eth_dev[i];
                netif_wake_queue(dev);
            }
            break;
    }
    return IFX_SUCCESS;
}

/*------------------------------------------------------------------------------------------*\
 * Allocates the buffer for ethernet packet.
 * This function is invoke when DMA callback function to be called 
 * to allocate a new buffer for Rx packets.
\*------------------------------------------------------------------------------------------*/
unsigned char* sw_dma_buffer_alloc(int len __attribute__ ((unused)),
                                   int* byte_offset,
                                   void** opt) {
    unsigned char  *buffer = NULL;
    struct sk_buff *skb    = NULL;
    int offset             = 0;
    int dma_burst_len      = g_dma_device->rx_burst_len << 2;

    *byte_offset = 2; /* for reserving 2 bytes in skb buffer, so, set offset 2 bytes infront of data pointer */
    skb = dev_alloc_skb(ETH_PKT_BUF_SIZE+dma_burst_len);
    if (skb == NULL) {
        printk(KERN_ERR "%s[%d]: Buffer allocation failed!!!\n", __func__,__LINE__);
        return NULL;
    }
    if(likely(skb)) {
        if(((u32)skb->data & (dma_burst_len -1)) != 0) {
            offset = ~((u32)skb->data+(dma_burst_len -1)) & (dma_burst_len -1);
        }
        if(offset != 0 ) {
            buffer = (unsigned char *)(skb->data+offset);
            skb_reserve(skb, offset);
        } else {
            buffer = (unsigned char*)(skb->data);
        }
        skb_reserve(skb, 2);
        *(int*)opt=(int)skb;
    }
    return buffer;
}

/*------------------------------------------------------------------------------------------*\
 * Free skb buf* Free skb buffer
 * This function frees a buffer previously allocated by the DMA buffer 
 *   alloc callback function. 
\*------------------------------------------------------------------------------------------*/
int sw_dma_buffer_free(unsigned char* dataptr,void* opt) {
    struct sk_buff *skb=NULL;
    
    if(opt==NULL) {
        if(dataptr)
            kfree(dataptr);
    } else {
        skb=(struct sk_buff*)opt;
        if(skb)
            dev_kfree_skb_any(skb);
    }
    
    return IFX_SUCCESS;
}

/*------------------------------------------------------------------------------------------*\
 * This function scheduled from upper layer when the NAPI is enabled
\*------------------------------------------------------------------------------------------*/
static int switch_poll(struct napi_struct *napi, int budget) {
	IFX_switch_priv_t *priv = container_of(napi, IFX_switch_priv_t, napi);
	struct net_device *dev = (struct net_device*)priv->owner;
    struct dma_device_info* dma_dev = g_dma_device;
    int work_done = 0;

    if (netif_running(dev)) {
        dma_device_poll(dma_dev, budget, &work_done);
    }

	return work_done;
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void eth_activate_poll(struct dma_device_info* dma_dev __attribute__ ((unused))) {
    //IFX_switch_priv_t *priv;
    struct net_device *dev;
    int i;
    for(i=0; i < NUM_ETH_INF ; i++) {
        if (!gphy_status[i].connected)
            continue;

        dev = ifx_eth_dev[i];
        ifx_eth_fw_poll_schedule(dev);
        //priv = netdev_priv(dev);
        //if ( netif_rx_schedule_prep(dev, &priv->napi) ) {
        //    __netif_rx_schedule(dev, &priv->napi);
        //    /*--- PRINT_AR9_DEBUG (KERN_ERR "[%s/%d] successful name %s\n", __FUNCTION__, __LINE__, dev->name); ---*/
        ///*--- } else { ---*/
        //    /*--- printk (KERN_ERR "[%s/%d] not successful\n", __FUNCTION__, __LINE__); ---*/
        //}
    }
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void eth_inactivate_poll(struct dma_device_info* dma_dev __attribute__ ((unused))) {
    //IFX_switch_priv_t *priv;
    struct net_device *dev;
    int i, success;

    /*--- PRINT_AR9_DEBUG (KERN_ERR "[%s/%d]\n", __FUNCTION__, __LINE__); ---*/
    success = 0;
    for(i=0; i < NUM_ETH_INF ; i++) {
        if (!gphy_status[i].connected)
            continue;

        dev = ifx_eth_dev[i];
        ifx_eth_fw_poll_schedule(dev);
        //priv = netdev_priv(dev);
        //if(netif_running(dev) ) {
        //    netif_rx_complete(dev, &priv->napi);
        //    success = 1;
        //    /*--- PRINT_AR9_DEBUG (KERN_ERR "[%s/%d] successful\n", __FUNCTION__, __LINE__); ---*/
        //} else {
        //    printk (KERN_ERR "[%s/%d] not successful\n", __FUNCTION__, __LINE__);
        //}
    }
    if(!success)
        return;
}


/*------------------------------------------------------------------------------------------*\
 * Unregister with DMA device core driver
\*------------------------------------------------------------------------------------------*/
static void dma_setup_uninit(void) {
    struct dma_device_info* dma_dev=g_dma_device;
    if( dma_dev ) {
        dma_device_unregister(dma_dev);
        dma_device_release(dma_dev);
    }
}

/*------------------------------------------------------------------------------------------*\
 * Register with DMA device core driver
\*------------------------------------------------------------------------------------------*/
static int dma_setup_init(void) {
    int i, ret;
    g_dma_device = dma_device_reserve("PPE");
    if(!g_dma_device) {
        printk(KERN_ERR "%s[%d]: Reserved with DMA core driver failed!!!\n", __func__,__LINE__);
        return -ENODEV;
    }
    g_dma_device->buffer_alloc              = &sw_dma_buffer_alloc;
    g_dma_device->buffer_free               = &sw_dma_buffer_free;
    g_dma_device->intr_handler              = &sw_dma_intr_handler;
    g_dma_device->num_rx_chan               = 4;
    g_dma_device->num_tx_chan               = 2;
    g_dma_device->tx_burst_len              = DMA_TX_BURST_LEN;
    g_dma_device->rx_burst_len              = DMA_RX_BURST_LEN;
    g_dma_device->tx_endianness_mode        = IFX_DMA_ENDIAN_TYPE3;
    g_dma_device->rx_endianness_mode        = IFX_DMA_ENDIAN_TYPE3;
    /*g_dma_device->port_packet_drop_enable   = IFX_DISABLE;*/
    
    for (i = 0; i < g_dma_device->num_rx_chan; i++) {
        g_dma_device->rx_chan[i]->packet_size =  ETH_PKT_BUF_SIZE;
        g_dma_device->rx_chan[i]->control = IFX_DMA_CH_ON;
    }
    for (i = 0; i < g_dma_device->num_tx_chan; i++) {
        /* TODO: Modify with application requirement 
        * eth0 -->DMA Tx channel 0, eth1 --> DMA Tx channel 1
        */
        if ( (i == 0) || (i == 1)) 
            g_dma_device->tx_chan[i]->control = IFX_DMA_CH_ON;
        else
            g_dma_device->tx_chan[i]->control = IFX_DMA_CH_OFF;
    }
    g_dma_device->activate_poll = eth_activate_poll;
    g_dma_device->inactivate_poll = eth_inactivate_poll;
    ret = dma_device_register (g_dma_device);
    if ( ret != IFX_SUCCESS)
        printk(KERN_ERR "%s[%d]: Register with DMA core driver Failed!!!\n", __func__,__LINE__);
        
    return ret;
}

/*------------------------------------------------------------------------------------------*\
 * init of the network device
\*------------------------------------------------------------------------------------------*/
static const struct net_device_ops netdev_ops = {
    .ndo_init            = switch_init,

    .ndo_open            = ifx_eth_open,
    .ndo_stop            = ifx_eth_close,
    .ndo_start_xmit      = ifx_start_xmit,
    .ndo_do_ioctl        = ifx_eth_ioctl,
    .ndo_get_stats       = ifx_get_stats,
    .ndo_change_mtu      = ifx_change_mtu,
    .ndo_set_mac_address = ifx_set_mac_address,
    .ndo_tx_timeout      = ifx_tx_timeout,
};
static int switch_init(struct net_device *dev) {
    u64 retval=0;
    static int macVal=0;
    int i;
    PRINT_AR9_DEBUG("[%s] %s up\n", __FUNCTION__, dev->name);

    dev->watchdog_timeo  = IFX_TX_TIMEOUT;

    SET_ETHTOOL_OPS(dev, &ifx_ethtool_ops);

    /*read the mac address from the mac table and put them into the mac table.*/
  	for (i = 0; i < 6; i++) {
		retval += my_ethaddr[i];
	}
	/* if ethaddr not set in u-boot, then use default one */
	if (retval == 0) {
	    dev->dev_addr[0] = 0x00;
		dev->dev_addr[1] = 0x20;
		dev->dev_addr[2] = 0xda;
		dev->dev_addr[3] = 0x86;
		dev->dev_addr[4] = 0x23;
		dev->dev_addr[5] = 0x74 + macVal;
	} else {
	    for (i = 0; i < 6; i++) {
	        dev->dev_addr[i] = my_ethaddr[i];
	    }
	    dev->dev_addr[5] += + macVal ;
	}
	macVal++;
    return 0;
}

/*------------------------------------------------------------------------------------------*\
 * Driver version info
\*------------------------------------------------------------------------------------------*/
static inline int eth_drv_ver(char *buf) {
    return sprintf(buf, "IFX 3 Port Switch Eth driver, version %s,(c)2009 Infineon Technologies AG\n", version);
}

/*------------------------------------------------------------------------------------------*\
 * Displays the version of ETH module via proc file
\*------------------------------------------------------------------------------------------*/
static int eth_proc_version(IFX_char_t *buf,
                            IFX_char_t **start __attribute__ ((unused)),
                            off_t offset __attribute__ ((unused)),
                            int count __attribute__ ((unused)), 
                            int *eof, 
                            void *data __attribute__ ((unused))) {
    int len = 0;
    /* No sanity check cos length is smaller than one page */
    len += eth_drv_ver(buf + len);
    *eof = 1;
    return len;    
}

/*------------------------------------------------------------------------------------------*\
 * create proc for debug  info, ifx_eth_module_init
\*------------------------------------------------------------------------------------------*/
static int eth_proc_create(void) {
    /* procfs */
    ifx_eth_dir = proc_mkdir ("driver/ifx_eth", NULL);
    if (ifx_eth_dir == NULL) {
        printk(KERN_ERR "%s: Create proc directory (/driver/ifx_eth) failed!!!\n", __func__);
        return IFX_ERROR;
    }
    create_proc_read_entry("version", 0, ifx_eth_dir, eth_proc_version,  NULL);
    return IFX_SUCCESS;
}

/*------------------------------------------------------------------------------------------*\
 * remove of the proc entries, ifx_eth_module_exit
\*------------------------------------------------------------------------------------------*/
static void eth_proc_delete(void) {
    remove_proc_entry("version", ifx_eth_dir);
    remove_proc_entry("driver/ifx_eth",  NULL);
}

#ifdef CONFIG_AVM_PA
static void ifx_eth_pa_dev_transmit(void *arg, struct sk_buff *skb) {
    skb->dev = (struct net_device *) arg;
    dev_queue_xmit(skb);
}
#endif

/*------------------------------------------------------------------------------------------*\
 * Init Ethernet module
\*------------------------------------------------------------------------------------------*/
int ifx_eth_module_init(void) {
    int i, err = 0;
    unsigned int reg;
    char ver_str[128] = {0};
    IFX_switch_priv_t* priv;
    char devicename[sizeof("ethx")];

    g_pmac_dma  = 0;
    g_dma_pmac  = 0;

    /** Register with DM core driver */
    dma_setup_init();

    init_MUTEX(&mdio_semaphore);

    ar9_3port_sw_hw_init();

    for (i = 0; i < NUM_ETH_INF ; i++) {
        if (!gphy_status[i].connected)
            continue;

        snprintf(devicename, sizeof("ethx"), "eth%d", i);
        printk("{%s} Device %s\n", __FUNCTION__, devicename);

        ifx_eth_dev[i] = alloc_netdev(sizeof(IFX_switch_priv_t), devicename, ether_setup);
        if (!ifx_eth_dev[i]) {
            printk(KERN_ERR "%s[%d]: no memory for ifx_eth_dev!!!\n", __func__,__LINE__);
            err = -ENOMEM;
            goto err_out_free_res;
        }
        ifx_eth_dev[i]->netdev_ops = &netdev_ops;
        priv = netdev_priv(ifx_eth_dev[i]);
        priv->owner = ifx_eth_dev[i];
        priv->phy_addr = gphy_status[i].phyaddr;
        /* By default, advertise supported  speed/duplex settings. */
#if 0
        priv->flags |= (IFX_FLAG_ADV_10HALF         \
                        | IFX_FLAG_ADV_10FULL       \
                        | IFX_FLAG_ADV_100HALF      \
                        | IFX_FLAG_ADV_100FULL      \
                        | IFX_FLAG_ADV_1000HALF     \
                        | IFX_FLAG_ADV_1000FULL)    \
                        ;
#else
        priv->flags |= gphy_status[i].flags;
#endif
        /* By default, auto-negotiate PAUSE. */
        priv->flags |= IFX_FLAG_PAUSE_AUTO;
        spin_lock_init(&priv->lock);
        priv->napi.state = 0;
        netif_napi_add(ifx_eth_dev[i], &priv->napi, switch_poll, TANTOS_NAPI_WEIGHT);

        if ( (err = register_netdev(ifx_eth_dev[i]))) {
            printk(KERN_ERR "%s[%d]: Register with network device failed!!!\n", __func__,__LINE__);
            goto err_out_free_res;
        }
#   ifdef CONFIG_AVM_PA
        {
        struct net_device *dev = ifx_eth_dev[i];
        {
            struct avm_pa_pid_cfg cfg;
            snprintf(cfg.name, sizeof(cfg.name), "%s", dev->name);
            cfg.framing = avm_pa_framing_ether;
            cfg.default_mtu = 1500;
            cfg.tx_func = ifx_eth_pa_dev_transmit;
            cfg.tx_arg = dev;
            if(avm_pa_dev_pid_register(AVM_PA_DEVINFO(dev), &cfg) < 0)
                printk(KERN_ERR "%s: failed to register PA PID\n", cfg.name);
        }
        {
            struct avm_pa_vpid_cfg cfg;
            snprintf(cfg.name, sizeof(cfg.name), "%s", dev->name);
            cfg.v4_mtu = 1500;
            cfg.v6_mtu = 1500;
            if(avm_pa_dev_vpid_register(AVM_PA_DEVINFO(dev), &cfg) < 0)
                printk(KERN_ERR "%s: failed to register PA VPID\n", cfg.name);
        }
        }
#   endif

    }
    reg = IFX_REG_R32(IFX_AR9_ETH_PMAC_HD_CTL );
    if (reg  & PMAC_HD_CTL_AS )
        g_pmac_dma = 1;
    if (reg  & PMAC_HD_CTL_RXSH )
        g_dma_pmac = 1;
    
    if (eth_proc_create() == IFX_ERROR ) 
        goto err_out_free_res;

    /* Print the driver version info */                      
    eth_drv_ver(ver_str);
    PRINT_AR9_DEBUG(KERN_INFO "%s", ver_str);
    return  0;
err_out_free_res:
    /*Unregister with DMA core driver */
    dma_setup_uninit();
    for (i=0; i< NUM_ETH_INF ; i++) {
        if (!gphy_status[i].connected)
            continue;

        /* unregister the network devices */
        if(ifx_eth_dev[i])
            free_netdev(ifx_eth_dev[i]);
    }
    return err;
}


/*------------------------------------------------------------------------------------------*\
 * 3 port ethernet module cleanup
 * Unregister with dma core driver and also network devices
\*------------------------------------------------------------------------------------------*/
void ifx_eth_module_exit(void) {
    int i;

    /* Un init of the GPIO's and free_irq*/
    ar9_3port_sw_hw_uninit();
    /*Unregister with DMA core driver */
    dma_setup_uninit();
    /* unregister the network devices */
    for (i=0; i< NUM_ETH_INF ; i++) {
        if (!gphy_status[i].connected)
            continue;

        unregister_netdev(ifx_eth_dev[i]);
        free_netdev(ifx_eth_dev[i]);
    }
    /** remove of the proc entries */
    eth_proc_delete();
}


/*------------------------------------------------------------------------------------------*\
 * moved from ifxmips_3port_eth_sw.h 
\*------------------------------------------------------------------------------------------*/
inline void ifx_mdelay(int delay __attribute__ ((unused))) {
    int i;
    for(i = delay; i > 0; i--)
        udelay(1000);
}


/*------------------------------------------------------------------------------------------*\
 * Reset of the Extern GPHY                                                                 *
\*------------------------------------------------------------------------------------------*/
void reset_ext_gphy(int gpio) {
    PRINT_AR9_DEBUG("[%s] Reset external PHY at GPIO %d\n", __FUNCTION__, gpio);

    ifx_gpio_output_set(gpio, IFX_GPIO_MODULE_EXTPHY_RESET);
    ifx_mdelay(500);
    ifx_gpio_output_clear(gpio, IFX_GPIO_MODULE_EXTPHY_RESET);
    ifx_mdelay(500);
    ifx_gpio_output_set(gpio, IFX_GPIO_MODULE_EXTPHY_RESET);
    ifx_mdelay(500);
}
EXPORT_SYMBOL(reset_ext_gphy);

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
unsigned int ar9_switch_read(unsigned int uRegAddr) {
    return IFX_REG_R32((volatile unsigned int*) (IFX_AR9_ETH_REG_BASE + uRegAddr));
}
EXPORT_SYMBOL(ar9_switch_read);

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
void ar9_switch_write(unsigned int uRegAddr, unsigned int data) {
    IFX_REG_W32(data, (volatile unsigned int*) (IFX_AR9_ETH_REG_BASE + uRegAddr));
}
EXPORT_SYMBOL(ar9_switch_write);

/*------------------------------------------------------------------------------------------*\
 * 3 port switch HW init
\*------------------------------------------------------------------------------------------*/
void switch_hw_init(int mode, int port) {
    unsigned int reg;
    int switchport = gphy_status[port].switchport;

    IFX_REG_W32(0, IFX_RCU_PPE_CONF);  
    
    reg = IFX_REG_R32(IFX_AR9_ETH_RGMII_CTL);
    printk("{%s} switchport %d reg 0x%x\n", __FUNCTION__, switchport, reg);
    if ( switchport == 0) {
        reg &= ~(RGMII_CTL_P0TDLY_MASK | RGMII_CTL_P0RDLY_MASK | RGMII_CTL_P0IS_MASK | RGMII_CTL_P0FRQ | RGMII_CTL_P0CKIO );
        /*Set RGMI0 Rx clock delay DLL 1.75nSec*/
        /*--- reg |= RGMII_CTL_P0RDLY_1_75; ---*/
        switch(mode) {
            case RGMII_MODE:
                reg |= (RGMII_CTL_P0IS_RGMII | RGMII_CTL_P0SPD_1000);
                break;
            case RGMII_MODE_100MB:
                reg |= (RGMII_CTL_P0IS_RGMII | RGMII_CTL_P0SPD_100);
                break;
            case RMII_MAC_MODE:
                reg |= (RGMII_CTL_P0IS_REDMII | RGMII_CTL_P0SPD_100);
                reg &= ~(RGMII_CTL_P0CKIO);
                break;
            default:
                printk(KERN_ERR "[%s] unsupported mode on phy %d\n", __FUNCTION__, switchport);
        }
    } else if (switchport == 1) {
        reg &= ~(RGMII_CTL_P1TDLY_MASK | RGMII_CTL_P1RDLY_MASK | RGMII_CTL_P1IS_MASK | RGMII_CTL_P1FRQ | RGMII_CTL_P1CKIO  );
        /*--- reg |= RGMII_CTL_P1TDLY_1_5 | RGMII_CTL_P1RDLY_1_5; ---*/
        switch(mode) {
            case RGMII_MODE:
                reg |= (RGMII_CTL_P1IS_RGMII | RGMII_CTL_P1SPD_1000);
                break;
            case RGMII_MODE_100MB:
                reg |= (RGMII_CTL_P1IS_RGMII | RGMII_CTL_P1SPD_100);
                break;
            case RMII_MAC_MODE:
                reg |= (RGMII_CTL_P1IS_REDMII | RGMII_CTL_P1SPD_100);
                reg &= ~(RGMII_CTL_P1CKIO | RGMII_CTL_P0CKIO);
                break;
            default:
                printk(KERN_ERR "[%s] unsupported mode on phy %d\n", __FUNCTION__, switchport);
        }
        printk("{%s} reg 0x%x\n", __FUNCTION__, reg);
    }

#if   defined( ENABLE_SWITCH_FLOW_CONTROL ) && ENABLE_SWITCH_FLOW_CONTROL
    reg |= RGMII_CTL_FCE_SET(switchport, 1);
#else
    reg |= RGMII_CTL_FCE_SET(switchport, 0);
#endif

    IFX_REG_W32( reg, IFX_AR9_ETH_RGMII_CTL);

    if (switchport)
        reg = IFX_REG_R32(IFX_AR9_ETH_P1_CTL );
    else
        reg = IFX_REG_R32(IFX_AR9_ETH_P0_CTL );

#if defined(INGRESS_DIRECT_FORWARD_ENABLE) && INGRESS_DIRECT_FORWARD_ENABLE
    reg |= PX_CTL_DFWD;
#else
    reg &= ~PX_CTL_DFWD;
#endif

    reg |= PX_CTL_BYPASS;
    /* Link is up during the interface up */
    /*reg |= PX_CTL_FLP; */
    reg |= (PX_CTL_IPVLAN | PX_CTL_SPE | PX_CTL_VPE);
    if (switchport == 1)
        IFX_REG_W32( reg, IFX_AR9_ETH_P1_CTL);
    else
        IFX_REG_W32( reg, IFX_AR9_ETH_P0_CTL);
}


/*------------------------------------------------------------------------------------------*\
 * Used for Interrupt Handler
\*------------------------------------------------------------------------------------------*/
static irqreturn_t mii1_gphy_int_handler(int irq __attribute__ ((unused)),
                                         void *dev_id __attribute__ ((unused))) {
    queue_work(mii1_gphy_workqueue, &mii1_gphy_work);
    disable_irq(MII1_GPHY_INT);
    return IRQ_HANDLED;
}

union phy_irq_status {
    struct _phy_irq_status {
        unsigned int reserved1 : 16;
        unsigned int wake_on_lan : 1;
        unsigned int msre : 1;
        unsigned int nprx : 1;
        unsigned int nptx : 1;
        unsigned int ane : 1;
        unsigned int anc : 1;
        unsigned int reserved2 : 4;
        unsigned int adsc : 1;
        unsigned int polariy : 1;
        unsigned int mdix : 1;
        unsigned int duplex : 1;
        unsigned int linkspeed : 1;
        unsigned int linkstate : 1;
    } Bits;
    unsigned int reg;
}; 

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static unsigned int phy11g_read_irq_status(unsigned int phyaddr) {

    return (unsigned int)ar9_switch_read_mdio(phyaddr, 0x1a);

}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static unsigned int ath8030_read_irq_status(unsigned int phyaddr) {

    unsigned int status;
    union phy_irq_status  phy_status;
   
    status = (unsigned int)ar9_switch_read_mdio(phyaddr, 0x13);

    printk("[%s] addr 0x%x status 0x%x ", __FUNCTION__, phyaddr, status);

    phy_status.reg = 0;
    if (status & (1<<15)) {
        printk("autoneg Error ");
        phy_status.Bits.linkstate = 1;
    }
    if (status & (1<<14)) {
        printk("linkspeed ");
        phy_status.Bits.linkspeed = 1;
    }
    if (status & (1<<13)) {
        printk("duplex ");
        phy_status.Bits.duplex = 1;
    }
    if (status & (1<<11)) {
        printk("autoneg complete ");
        phy_status.Bits.linkstate = 1;
    }
    if (status & (1<<10)) {
        printk("linkstate ");
        phy_status.Bits.linkstate = 1;
    }
    if (status & (1<<8)) {
        printk("carrier false ");
        phy_status.Bits.linkstate = 1;
    }
    if (status & (1<< 6)) {
        printk("mdix ");
        phy_status.Bits.mdix = 1;
    }

    printk("phy_status 0x%x\n", phy_status.reg);

    return phy_status.reg;
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static unsigned int phy11g_read_link_status(unsigned int phyaddr) {

    unsigned int status = ar9_switch_read_mdio(phyaddr, 0x18);

    return status;
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static unsigned int ath8030_read_link_status(unsigned int phyaddr) {

    unsigned int tmp = ar9_switch_read_mdio(phyaddr, 0x11);
    unsigned int status = 0;

    printk("{%s} phyaddr 0x%x status 0x%x\n", __FUNCTION__, phyaddr, status);

    status |= (tmp >> 14);      /*--- set Speed ---*/
    if ((tmp & (1<<11)) && (tmp & (1<<13)))
        status |= (1<<3);       /*--- set duplex ---*/

    return status;
}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void phy11g_config_phy(unsigned int phyaddr) {

    ar9_switch_write_mdio(phyaddr, 0x19, 
            (1 << 0) |   /*--- Link State change mask (enable) ---*/
            (1 << 1) |   /*--- Link Speed change mask (enable) ---*/
            (1 << 2) |   /*--- Link Duplex change mask (enable) ---*/
            (1 << 3) |   /*--- Link MDIX change mask (enable) ---*/
            (1 << 4) |   /*--- Link MDI Polarity change mask (enable) ---*/
            /*--- (1 << 5) | ---*/   /*--- Link Speed auto down shift detect mask (enable) ---*/
            /*--- (1 << 10) | ---*/   /*--- auto neg complete mask (enable) ---*/
            /*--- (1 << 11) | ---*/   /*--- auto neg error mask (enable) ---*/
            /*--- (1 << 12) | ---*/   /*--- next page transmit mask (enable) ---*/
            /*--- (1 << 13) | ---*/   /*--- next page receive mask (enable) ---*/
            /*--- (1 << 14) | ---*/   /*--- master slave resolution error mask (enable) ---*/
            /*--- (1 << 15) | ---*/   /*--- wake on LAN Event mask (enable) ---*/
            0);
    ar9_switch_write_mdio(phyaddr, 0x0, 
            /*--- (1 << 6) | ---*/   /*--- force speed selection (MSB) ---*/
            /*--- (1 << 7) | ---*/   /*--- collition test duplex ---*/
            (1 << 8) |   /*--- full duplex ---*/
            (1 << 9) |   /*--- restart autoneg ---*/
            /*--- (1 << 10) | ---*/   /*--- isolate ---*/
            /*--- (1 << 11) | ---*/   /*--- power down ---*/
            (1 << 12) |   /*--- autoneg enable ---*/
            /*--- (1 << 13) | ---*/   /*--- force speed selection (LSB) ---*/
            /*--- (1 << 14) | ---*/   /*--- loop back ---*/
            /*--- (1 << 15) | ---*/   /*--- reset ---*/
            0);

}

/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void ath8030_config_phy(unsigned int phyaddr) {

    printk("{%s} phyaddr 0x%x\n", __FUNCTION__, phyaddr);

    ar9_switch_write_mdio(phyaddr, 0x12, 
            /*--- (1 << 0) | ---*/   
            (1 << 1) |   /*--- Link MDI Polarity change mask (enable) ---*/
            /*--- (1 << 2) | ---*/   
            /*--- (1 << 3) | ---*/   
            /*--- (1 << 4) | ---*/   
            /*--- (1 << 5) | ---*/   /*--- Link Speed auto down shift detect mask (enable) ---*/
            (1 << 6)  |  /*--- Link MDIX change mask (enable) ---*/
            (1 << 8)  |  /*--- False Carrier mask (enable) ---*/
            (1 << 10) |  /*--- Link State change mask (enable) ---*/ 
            (1 << 11) |   /*--- auto neg complete mask (enable) ---*/
            /*--- (1 << 12) | ---*/   /*--- next page transmit mask (enable) ---*/
            (1 << 13) |  /*--- Link Duplex change mask (enable) ---*/ 
            (1 << 14) |  /*--- Link Speed change mask (enable) ---*/ 
            (1 << 15) |   /*--- Autoneg-Error mask (enable) ---*/
            0);

    ar9_switch_write_mdio(phyaddr, 0x0, 
            /*--- (1 << 6) | ---*/   /*--- force speed selection (MSB) ---*/
            /*--- (1 << 7) | ---*/   /*--- collition test duplex ---*/
            (1 << 8) |   /*--- full duplex ---*/
            (1 << 9) |   /*--- restart autoneg ---*/
            /*--- (1 << 10) | ---*/   /*--- isolate ---*/
            /*--- (1 << 11) | ---*/   /*--- power down ---*/
            (1 << 12) |   /*--- autoneg enable ---*/
            /*--- (1 << 13) | ---*/   /*--- force speed selection (LSB) ---*/
            /*--- (1 << 14) | ---*/   /*--- loop back ---*/
            /*--- (1 << 15) | ---*/   /*--- reset ---*/
            0);

    printk("{ath8030 reg 0x0 = 0x%x 0x12 = 0x%x}\n", ar9_switch_read_mdio(phyaddr, 0), ar9_switch_read_mdio(phyaddr, 0x12));
}


/*------------------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------------------*/
static void mii1_gphy_work_handler(struct work_struct *work __attribute__ ((unused))) {
    unsigned short  linkReg, speedReg;
    unsigned int reg_port;
    unsigned int port;
    /*--- unsigned int addr; ---*/
    union phy_irq_status  status;

#if 0
    union __11g_irq_status {
        struct _11g_irq_status {
            unsigned int reserved1 : 16;
            unsigned int wake_on_lan : 1;
            unsigned int msre : 1;
            unsigned int nprx : 1;
            unsigned int nptx : 1;
            unsigned int ane : 1;
            unsigned int anc : 1;
            unsigned int reserved2 : 4;
            unsigned int adsc : 1;
            unsigned int mdipc : 1;
            unsigned int mdixc : 1;
            unsigned int dxmc : 1;
            unsigned int lspc : 1;
            unsigned int lstc : 1;
        } Bits;
        unsigned int reg;
    } status;
#endif

    for(port = 0 ; port < NUM_ETH_INF ; port++) {
        int old = gphy_status[port].link;
        
        if (!gphy_status[port].connected)
            continue;

        /*--- status.reg = ar9_switch_read_mdio(addr, 0x1a); ---*/ 
        status.reg = gphy_status[port].read_phy_int_status(gphy_status[port].phyaddr);
    
        PRINT_AR9_DEBUG(KERN_ERR "[%s] port %d status 0x%x\n", __FUNCTION__, port, status.reg);

        /*--- interrupt status bit ---*/
        if(status.Bits.linkstate || status.Bits.linkspeed || status.Bits.duplex || status.Bits.mdix) { 

            /*cause the interrupt due to link status, speed & FDX */
            linkReg = ar9_switch_read_mdio(gphy_status[port].phyaddr, 0x01);     /*--- default-Phy-register ---*/

            /*Link on PHY is down */
            /*--- reg_port = IFX_REG_R32(port == 0 ? IFX_AR9_ETH_P0_CTL : IFX_AR9_ETH_P1_CTL); ---*/
            reg_port = IFX_REG_R32(gphy_status[port].switchport == 0 ? IFX_AR9_ETH_P0_CTL : IFX_AR9_ETH_P1_CTL);

            /*------------------------------------------------------------------------------*\
             * Link Status prüfen
            \*------------------------------------------------------------------------------*/
            if((linkReg & (1 << 2)) == 0) {  /*--- link Status ---*/
                reg_port &= ~(PX_CTL_FLD|PX_CTL_FLP);
                /*Port1 force link down */
                reg_port |= PX_CTL_FLD;
                gphy_status[port].link = 0x0; /*LINK DOWN */
            } else {
                union __port_control_register {
                    struct _port_control_register {
                        unsigned int mcs : 8;
                        unsigned int p1_clk_select : 1;
                        unsigned int p1_clk_freq : 1;
                        unsigned int p0_clk_select : 1;
                        unsigned int p0_clk_freq : 1;
                        unsigned int p1_interface_mode : 2;
                        unsigned int p1_rx_clk_delay : 2;
                        unsigned int p1_tx_clk_delay : 2;
                        unsigned int p1_speed : 2;
                        unsigned int p1_duplex : 1;
                        unsigned int p1_flow_enable : 1;
                        unsigned int p0_interface_mode : 2;
                        unsigned int p0_rx_clk_delay : 2;
                        unsigned int p0_tx_clk_delay : 2;
                        unsigned int p0_speed : 2;
                        unsigned int p0_duplex : 1;
                        unsigned int p0_flow_enable : 1;
                    } Bits;
                    unsigned int reg;
                } reg_rgmii_save, reg_rgmii;

                /*--------------------------------------------------------------------------*\
                 * Speed Prüfen wenn link UP
                \*--------------------------------------------------------------------------*/
                speedReg = gphy_status[port].read_phy_link_status(gphy_status[port].phyaddr);
                reg_rgmii.reg = IFX_REG_R32(IFX_AR9_ETH_RGMII_CTL);
                reg_rgmii_save = reg_rgmii;
                reg_rgmii.reg >>= port * 10;  /*--- hierdurch kann ich immer die p0_... bits nutzen ---*/

                if(speedReg & (1 << 3)) {
                    reg_rgmii.Bits.p0_duplex = 1;
                    gphy_status[port].duplex = DUPLEX_FULL;
                }else {
                    reg_rgmii.Bits.p0_duplex = 0;
                    gphy_status[port].duplex = DUPLEX_HALF;
                }
                switch (speedReg & (3 << 0)) {
                    case 0 :  /*speed 10BASE-T */
                        reg_rgmii.Bits.p0_speed = 0;
                        reg_port &= ~(PX_CTL_FLD|PX_CTL_FLP);
                        /*Port1 force link up*/
#if defined(CONFIG_PMAC_DMA_ENABLE) && CONFIG_PMAC_DMA_ENABLE
                        if(gphy_status[port].eth_up)
                            reg_port |= PX_CTL_FLP;
#else
                        reg_port |= PX_CTL_FLP;
#endif 
                        gphy_status[port].speed = SPEED_10;
                        gphy_status[port].link = 0x1; /*LINK UP */
                        PRINT_AR9_DEBUG(KERN_ERR "[%s] speed = %d duplex = %s\n", __FUNCTION__, 10, gphy_status[port].duplex == DUPLEX_HALF ? "half" : "full");
                        break;
                    case 1:	  /*speed 100BASE-TX*/
                        reg_rgmii.Bits.p0_speed = 1;
                        reg_port &= ~(PX_CTL_FLD|PX_CTL_FLP);
                        /*Port1 force link up*/
#if defined(CONFIG_PMAC_DMA_ENABLE) && CONFIG_PMAC_DMA_ENABLE
                        if(gphy_status[port].eth_up)
                            reg_port |= PX_CTL_FLP;
#else
                        reg_port |= PX_CTL_FLP;
#endif 
                        gphy_status[port].speed = SPEED_100;
                        gphy_status[port].link = 0x1; /*LINK UP */
                        PRINT_AR9_DEBUG(KERN_ERR "[%s] speed = %d duplex = %s\n", __FUNCTION__, 100, gphy_status[port].duplex == DUPLEX_HALF ? "half" : "full");
                        break;
                    case 2:	  /*speed 1000BASE-T*/
                        reg_rgmii.Bits.p0_speed = 2;
                        reg_port &= ~(PX_CTL_FLD|PX_CTL_FLP);
                        /*Port1 force link up*/
#if defined(CONFIG_PMAC_DMA_ENABLE) && CONFIG_PMAC_DMA_ENABLE
                        if(gphy_status[port].eth_up)
                            reg_port |= PX_CTL_FLP;
#else
                        reg_port |= PX_CTL_FLP;
#endif 
                        gphy_status[port].speed = SPEED_1000;
                        gphy_status[port].link = 0x1; /*LINK UP */
                        PRINT_AR9_DEBUG(KERN_ERR "[%s] speed = %d duplex = %s\n", __FUNCTION__, 1000, gphy_status[port].duplex == DUPLEX_HALF ? "half" : "full");
                        break;
                }
                if(gphy_status[port].switchport == 0) {
                    IFX_REG_W32( reg_rgmii.reg, IFX_AR9_ETH_RGMII_CTL);
                } else {
                    reg_rgmii_save.Bits.p1_speed = reg_rgmii.Bits.p0_speed;
                    reg_rgmii_save.Bits.p1_duplex = reg_rgmii.Bits.p0_duplex;
                    IFX_REG_W32( reg_rgmii_save.reg, IFX_AR9_ETH_RGMII_CTL);
                }
                reg_rgmii.reg = IFX_REG_R32(IFX_AR9_ETH_RGMII_CTL);
                PRINT_AR9_DEBUG(KERN_ERR "[%s] IFX_AR9_ETH_RGMII_CTL 0x%x\n", __FUNCTION__, reg_rgmii.reg);
            }
            IFX_REG_W32( reg_port, gphy_status[port].switchport == 0 ? IFX_AR9_ETH_P0_CTL : IFX_AR9_ETH_P1_CTL);
            PRINT_AR9_DEBUG(KERN_ERR "[%s] %s 0x%x\n", __FUNCTION__, gphy_status[port].switchport == 0 ? "IFX_AR9_ETH_P0_CTL" : "IFX_AR9_ETH_P1_CTL", reg_port);
            cpmac_interface_port_update((unsigned char) gphy_status[port].switchport, 
                                        (unsigned char) gphy_status[port].link,
                                        (gphy_status[port].speed == SPEED_1000) ? avm_event_ethernet_speed_1G 
                                        : ((gphy_status[port].speed == SPEED_100) ? avm_event_ethernet_speed_100M 
                                        : avm_event_ethernet_speed_10M),
                                        gphy_status[port].duplex == DUPLEX_FULL);
        }
        if(old != gphy_status[port].link) {
            if(gphy_status[port].dev) {
                if(gphy_status[port].link) {
                    netif_carrier_on(gphy_status[port].dev);
                    netif_start_queue(gphy_status[port].dev);
                } else {
                    netif_carrier_off(gphy_status[port].dev);
                    netif_stop_queue(gphy_status[port].dev);
                }
                if (netif_carrier_ok(gphy_status[port].dev)) {
                    PRINT_AR9_DEBUG(KERN_ERR "[%s] '%s' link is up\n",   __FUNCTION__, gphy_status[port].dev->name);
                } else {
                    PRINT_AR9_DEBUG(KERN_ERR "[%s] '%s' link is down\n", __FUNCTION__, gphy_status[port].dev->name);
                } 
            }
        }
    }
    enable_irq(MII1_GPHY_INT);

#if 0
    { 
        unsigned int wert;
#define IFX_AR9_SW_GCTL0   (0xBE108000   + 0x68)
        wert = IFX_REG_R32( IFX_AR9_SW_GCTL0 );
        PRINT_AR9_DEBUG(KERN_ERR "[%s] IFX_AR9_SW_GCTL0 0x%x (LPE %d)\n", __FUNCTION__, wert, wert >> 29 ? 1 : 0);
#define IFX_AR9_PMAC_HD_CTL (0xBE108000   + 0x8C)
     
        wert = IFX_REG_R32( IFX_AR9_PMAC_HD_CTL );
        PRINT_AR9_DEBUG(KERN_ERR "[%s] IFX_AR9_PMAC_HD_CTL 0x%x (AS %d, AC %d)\n", __FUNCTION__, wert, wert >> 19 ? 1 : 0, wert >> 18 ? 1 : 0);
    }
#endif
}



/*------------------------------------------------------------------------------------------*\
 * Init of the 3 port switch
\*------------------------------------------------------------------------------------------*/
void ar9_3port_sw_hw_uninit(void) {
	ifx_gpio_output_clear(RESET_GPHY0, IFX_GPIO_MODULE_EXTPHY_RESET);
#if defined( CONFIG_IFX_MAC1) && CONFIG_IFX_MAC1
    /*GPIO P2.0*/
	ifx_gpio_output_clear(RESET_GPHY1, IFX_GPIO_MODULE_EXTPHY_RESET);
#endif /** CONFIG_IFX_MAC1 */
    free_irq(MII1_GPHY_INT, (void*)&gphy_status);

}


/*------------------------------------------------------------------------------------------*\
 * Init of the 3 port switch
\*------------------------------------------------------------------------------------------*/
int ar9_3port_sw_hw_init(void) {
    unsigned int reg;
    int result = 0;
    int port = 0, addr;

	ifx_gpio_register(IFX_GPIO_MODULE_EXTPHY_INT);
	ifx_gpio_register(IFX_GPIO_MODULE_EXTPHY_MDIO);
	ifx_gpio_register(IFX_GPIO_MODULE_EXTPHY_RESET);

    SWITCH_PMU_SETUP(IFX_PMU_ENABLE);

    /**Enable Switch  */
    IFX_REG_W32 ( (IFX_REG_R32(IFX_AR9_ETH_GCTL0) | GCTL0_SE), IFX_AR9_ETH_GCTL0) ;
    IFX_REG_W32 ( (IFX_REG_R32(IFX_AR9_ETH_P0_CTL) | PX_CTL_DMDIO), IFX_AR9_ETH_P0_CTL) ;
    IFX_REG_W32 ( (IFX_REG_R32(IFX_AR9_ETH_P1_CTL) | PX_CTL_DMDIO), IFX_AR9_ETH_P1_CTL) ;
    
    /*------------------------------------------------------------------------------------------*\
     * automatische PHY Erkennung
    \*------------------------------------------------------------------------------------------*/
    reset_ext_gphy(GPIO_RESET_GPHY0);
    reset_ext_gphy(GPIO_RESET_GPHY1);

    for(addr = 0 ; addr < 31 ; addr++) {        /*--- alle möglichen PHY-Adressen scannen ---*/
        unsigned short phy_id1, phy_id2;
        phy_id1 = ar9_switch_read_mdio(addr, 0x2);
        phy_id2 = ar9_switch_read_mdio(addr, 0x3);

        if (phy_id1 != NO_PHY_ID)
            printk(KERN_ERR "[%s] phy %d id: 0x%04x : 0x%04x\n", __FUNCTION__, addr, phy_id1, phy_id2);

        switch (phy_id1) {
            case PHY11G_CHIP_ID:
                gphy_status[port].connected = 1;
                gphy_status[port].phyaddr = addr;
                gphy_status[port].read_phy_int_status = phy11g_read_irq_status;
                gphy_status[port].read_phy_link_status = phy11g_read_link_status;
                gphy_status[port].config_phy = phy11g_config_phy;
                gphy_status[port].switchport = port;
                gphy_status[port].flags = (   IFX_FLAG_ADV_10HALF | IFX_FLAG_ADV_10FULL   \
                                            | IFX_FLAG_ADV_100HALF | IFX_FLAG_ADV_100FULL \
                                            | IFX_FLAG_ADV_1000HALF | IFX_FLAG_ADV_1000FULL);
                switch_hw_init(MII0_MODE_SETUP, port);
                port++;
                break;
            case ATH8030_CHIP_ID:
                port = 1;     /*--- ist an Port 1 ---*/
                gphy_status[port].connected = 1;
                gphy_status[port].phyaddr = addr;
                gphy_status[port].read_phy_int_status = ath8030_read_irq_status;
                gphy_status[port].read_phy_link_status = ath8030_read_link_status;
                gphy_status[port].config_phy = ath8030_config_phy;
                gphy_status[port].switchport = port;
                gphy_status[port].flags = (   IFX_FLAG_ADV_10HALF | IFX_FLAG_ADV_10FULL   \
                                            | IFX_FLAG_ADV_100HALF | IFX_FLAG_ADV_100FULL);
                switch_hw_init(RMII_MAC_MODE, port);
                port++;
                break;
            case NO_PHY_ID:
                gphy_status[port].connected = 0;
                break;
            default:
                printk(KERN_ERR "[%s] ERROR unknown PHY 0x%04x 0x%04x on Port %d Address %d\n", __FUNCTION__, phy_id1, phy_id2, port, addr);
        }
    }
#if 0
    /*--------------------------------------------------------------------------------------*\
    \*--------------------------------------------------------------------------------------*/
    /** Disable MDIO auto polling mode */
    PRINT_AR9_DEBUG(KERN_ERR "[%s] reset eth0\n", __FUNCTION__);
    switch_hw_init(MII0_MODE_SETUP, 0);
    reset_ext_gphy(GPIO_RESET_GPHY0);

#if defined( CONFIG_IFX_MAC1) && CONFIG_IFX_MAC1
    PRINT_AR9_DEBUG(KERN_ERR "[%s] reset eth1\n", __FUNCTION__);
    /*--- switch_hw_init(MII1_MODE_SETUP, 1); ---*/
    switch_hw_init(RMII_MAC_MODE, 1);
    /*--- reset_ext_gphy(GPIO_RESET_GPHY1); ---*/
#endif /** CONFIG_IFX_MAC1 */
#endif

    reg = IFX_REG_R32(IFX_AR9_ETH_GCTL0 );
    reg &= ~GCTL0_MPL_MASK;
    reg |= GCTL0_MPL_1522;
    IFX_REG_W32( reg, IFX_AR9_ETH_GCTL0);
    
    reg  = IFX_REG_R32(IFX_AR9_ETH_P2_CTL );
    reg |= PX_CTL_BYPASS;
    reg |= PX_CTL_FLP;
    reg |= (PX_CTL_IPVLAN | PX_CTL_SPE | PX_CTL_VPE);
    IFX_REG_W32( reg, IFX_AR9_ETH_P2_CTL);
    reg = IFX_REG_R32(IFX_AR9_ETH_PMAC_HD_CTL );
    reg |= PMAC_HD_CTL_AC;
#if defined(CONFIG_PMAC_DMA_ENABLE) && CONFIG_PMAC_DMA_ENABLE
    reg |= PMAC_HD_CTL_AS;
#else
    reg &= ~PMAC_HD_CTL_AS;
#endif
#if defined(CONFIG_DMA_PMAC_ENABLE) && CONFIG_DMA_PMAC_ENABLE
    reg |= PMAC_HD_CTL_RXSH;
#else
    reg &= ~PMAC_HD_CTL_RXSH;
#endif
    IFX_REG_W32( reg, IFX_AR9_ETH_PMAC_HD_CTL);
    
	/*------------------------------------------------------------------------------------------*\
	 *  Fix von Lantiq um das Vollaufen des TX-DMA-Puffers zu vermeiden
	\*------------------------------------------------------------------------------------------*/
    IFX_REG_W32(0x13B, IFX_AR9_ETH_PMAC_RX_IPG);

    /*Queue Mapping */
    IFX_REG_W32( 0xFA41, IFX_AR9_ETH_1P_PRT);
    IFX_REG_W32( 0x11100000, IFX_AR9_ETH_DFSRV_MAP0);
    IFX_REG_W32( 0x22211111, IFX_AR9_ETH_DFSRV_MAP1);
    IFX_REG_W32( 0x30222222, IFX_AR9_ETH_DFSRV_MAP2);
    IFX_REG_W32( 0x00030003, IFX_AR9_ETH_DFSRV_MAP3);
    
/** Scheduling Scheme
**  Strict Priority scheduling is used for all ports and Rate limit is disabled*/
    IFX_REG_W32( 0x03E803E8, IFX_AR9_ETH_P0_ECS_Q32_REG);
    IFX_REG_W32( 0x03E803E8, IFX_AR9_ETH_P0_ECS_Q10_REG);
    IFX_REG_W32( 0x03E803E8, IFX_AR9_ETH_P1_ECS_Q32_REG);
    IFX_REG_W32( 0x03E803E8, IFX_AR9_ETH_P1_ECS_Q10_REG);
    IFX_REG_W32( 0x03E803E8, IFX_AR9_ETH_P2_ECS_Q32_REG);
    IFX_REG_W32( 0x03E803E8, IFX_AR9_ETH_P2_ECS_Q10_REG);
    
    IFX_REG_W32( 0, IFX_AR9_ETH_P0_ECW_Q32_REG);
    IFX_REG_W32( 0, IFX_AR9_ETH_P0_ECW_Q10_REG);
    IFX_REG_W32( 0, IFX_AR9_ETH_P1_ECW_Q32_REG);
    IFX_REG_W32( 0, IFX_AR9_ETH_P1_ECW_Q10_REG);
    IFX_REG_W32( 0, IFX_AR9_ETH_P2_ECW_Q32_REG);
    IFX_REG_W32( 0, IFX_AR9_ETH_P2_ECW_Q10_REG);

    for(port = 0 ; port < NUM_ETH_INF ; port++) {
        printk("{%s} port %d %s connected\n", __FUNCTION__, port, !gphy_status[port].connected ? "not":"");
        if (!gphy_status[port].connected)
            continue;
        ar9_switch_write_mdio(gphy_status[port].phyaddr, 0x0, 
                /*--- (1 << 6) | ---*/   /*--- force speed selection (MSB) ---*/
                /*--- (1 << 7) | ---*/   /*--- collition test duplex ---*/
                (1 << 8) |   /*--- full duplex ---*/
                /*--- (1 << 9) | ---*/   /*--- restart autoneg ---*/
                /*--- (1 << 10) | ---*/   /*--- isolate ---*/
                /*--- (1 << 11) | ---*/   /*--- power down ---*/
                (1 << 12) |   /*--- autoneg enable ---*/
                /*--- (1 << 13) | ---*/   /*--- force speed selection (LSB) ---*/
                /*--- (1 << 14) | ---*/   /*--- loop back ---*/
                (1 << 15) |   /*--- reset ---*/
                0);
        while (ar9_switch_read_mdio(gphy_status[port].phyaddr, 0x0) & (1<<15)) ;
        PRINT_AR9_DEBUG("<RESET done>\n");
    }

    mii1_gphy_workqueue = create_singlethread_workqueue("mii1_gphy_workqueue");
    if(mii1_gphy_workqueue == NULL) {
        printk(KERN_ERR "[%s] create_singlethread_workqueue(\"mii1_gphy_workqueue\" failed\n", __FUNCTION__);
        return -EFAULT;
    }

    result = request_irq(MII1_GPHY_INT, mii1_gphy_int_handler, IRQF_DISABLED | IRQF_TRIGGER_LOW, "ifx_mii1_gphy", (void*)gphy_status);
    /*--- result = request_irq(MII1_GPHY_INT, mii1_gphy_int_handler, IRQF_DISABLED | IRQF_TRIGGER_HIGH, "ifx_mii1_gphy", (void*)gphy_status); ---*/
    if ( result ){
        printk(KERN_ERR "[%s] ERROR cannot get ext %d irq!\n", __FUNCTION__, MII1_GPHY_INT);
        return -EFAULT;
    }

    for(port = 0 ; port < NUM_ETH_INF ; port++) {
        if (!gphy_status[port].connected)
            continue;

        printk(KERN_ERR "[%s] config phy port %d address %d\n", __FUNCTION__, port, gphy_status[port].phyaddr);
        gphy_status[port].config_phy(gphy_status[port].phyaddr);

        printk(KERN_ERR "[%s] phy %d id: 0x%04x : 0x%04x\n", __FUNCTION__, gphy_status[port].phyaddr, 
                ar9_switch_read_mdio(gphy_status[port].phyaddr, 0x2), 
                ar9_switch_read_mdio(gphy_status[port].phyaddr, 0x3));

    }

    return result;
}

/*------------------------------------------------------------------------------------------*\
 * moved from ifxmips_3port_eth_sw.h end
\*------------------------------------------------------------------------------------------*/

module_init(ifx_eth_module_init);
module_exit(ifx_eth_module_exit);

MODULE_AUTHOR("Reddy Mallikarjuna");
MODULE_DESCRIPTION("IFX ethernet driver (Supported AR9)");
MODULE_LICENSE("GPL");
MODULE_VERSION(IFX_DRV_MODULE_VERSION);
